MODULE mod_huffman
IMPLICIT NONE
PRIVATE 
PUBLIC :: init_huffman, init_huffman_quadruples ! subroutine
PUBLIC :: huff_, huffq, huff_qa, huff_qb, huff  ! variable
TYPE :: huff_
 INTEGER :: nmax
 INTEGER :: linbits
 INTEGER :: linmax
 INTEGER, POINTER :: leng(:, :) ! ALLOCATABLE
 INTEGER, POINTER :: icod(:, :) ! ALLOCATABLE
END TYPE  huff_
!
TYPE :: huffq
 INTEGER :: leng, icod
END TYPE huffq
!
TYPE (huffq), SAVE :: huff_qa(0:1,0:1,0:1,0:1), huff_qb(0:1,0:1,0:1,0:1)
TYPE (huff_), SAVE :: huff(0:31)
CONTAINS
!----------------------------------------------------------------------------------------- 
SUBROUTINE init_huffman()
IMPLICIT NONE
huff(0)%nmax    = 0
huff(0)%linbits = 0
CALL alloc_huff( huff(0) )
CALL sub_init_huff(0, 0, '', huff(0) )
!
huff(1)%nmax    = 1
huff(1)%linbits = 0
CALL alloc_huff( huff(1) )
CALL sub_init_huff(0, 0,   '1', huff(1) )
CALL sub_init_huff(0, 1, '001', huff(1) )
CALL sub_init_huff(1, 0,  '01', huff(1) )
CALL sub_init_huff(1, 1, '000', huff(1) )
!
huff(2)%nmax    = 2
huff(2)%linbits = 0
CALL alloc_huff( huff(2) )
CALL sub_init_huff(0, 0,      '1', huff(2) )
CALL sub_init_huff(0, 1,    '010', huff(2) )
CALL sub_init_huff(0, 2, '000001', huff(2) )
CALL sub_init_huff(1, 0,    '011', huff(2) )
CALL sub_init_huff(1, 1,    '001', huff(2) )
CALL sub_init_huff(1, 2,  '00001', huff(2) )
CALL sub_init_huff(2, 0,  '00011', huff(2) )
CALL sub_init_huff(2, 1,  '00010', huff(2) )
CALL sub_init_huff(2, 2, '000000', huff(2) )
!
huff(3)%nmax    = 2
huff(3)%linbits = 0
CALL alloc_huff( huff(3) )
CALL sub_init_huff(0, 0,     '11', huff(3) )
CALL sub_init_huff(0, 1,     '10', huff(3) )
CALL sub_init_huff(0, 2, '000001', huff(3) )
CALL sub_init_huff(1, 0,    '001', huff(3) )
CALL sub_init_huff(1, 1,     '01', huff(3) )
CALL sub_init_huff(1, 2,  '00001', huff(3) )
CALL sub_init_huff(2, 0,  '00011', huff(3) )
CALL sub_init_huff(2, 1,  '00010', huff(3) )
CALL sub_init_huff(2, 2, '000000', huff(3) )
!
huff(4)%nmax    = -1
huff(4)%linbits =  0
huff(4)%linmax  = -1
!
huff(5)%nmax    = 3
huff(5)%linbits = 0
CALL alloc_huff( huff(5) )
CALL sub_init_huff(0, 0,        '1', huff(5) )
CALL sub_init_huff(0, 1,      '010', huff(5) )
CALL sub_init_huff(0, 2,   '000110', huff(5) )
CALL sub_init_huff(0, 3,  '0000101', huff(5) )
CALL sub_init_huff(1, 0,      '011', huff(5) )
CALL sub_init_huff(1, 1,      '001', huff(5) )
CALL sub_init_huff(1, 2,   '000100', huff(5) )
CALL sub_init_huff(1, 3,  '0000100', huff(5) )
CALL sub_init_huff(2, 0,   '000111', huff(5) )
CALL sub_init_huff(2, 1,   '000101', huff(5) )
CALL sub_init_huff(2, 2,  '0000111', huff(5) )
CALL sub_init_huff(2, 3, '00000001', huff(5) )
CALL sub_init_huff(3, 0,  '0000110', huff(5) )
CALL sub_init_huff(3, 1,   '000001', huff(5) )
CALL sub_init_huff(3, 2,  '0000001', huff(5) )
CALL sub_init_huff(3, 3, '00000000', huff(5) )
!
huff(6)%nmax    = 3
huff(6)%linbits = 0
CALL alloc_huff( huff(6) )
CALL sub_init_huff(0, 0,     '111', huff(6) )
CALL sub_init_huff(0, 1,     '011', huff(6) )
CALL sub_init_huff(0, 2,   '00101', huff(6) )
CALL sub_init_huff(0, 3, '0000001', huff(6) )
CALL sub_init_huff(1, 0,     '110', huff(6) )
CALL sub_init_huff(1, 1,      '10', huff(6) )
CALL sub_init_huff(1, 2,    '0011', huff(6) )
CALL sub_init_huff(1, 3,   '00010', huff(6) )
CALL sub_init_huff(2, 0,    '0101', huff(6) )
CALL sub_init_huff(2, 1,    '0100', huff(6) )
CALL sub_init_huff(2, 2,   '00100', huff(6) )
CALL sub_init_huff(2, 3,  '000001', huff(6) )
CALL sub_init_huff(3, 0,  '000011', huff(6) )
CALL sub_init_huff(3, 1,   '00011', huff(6) )
CALL sub_init_huff(3, 2,  '000010', huff(6) )
CALL sub_init_huff(3, 3, '0000000', huff(6) )
!
huff(7)%nmax    = 5
huff(7)%linbits = 0
CALL alloc_huff( huff(7) )
CALL sub_init_huff(0, 0,         '1', huff(7) )
CALL sub_init_huff(0, 1,       '010', huff(7) )
CALL sub_init_huff(0, 2,    '001010', huff(7) )
CALL sub_init_huff(0, 3,  '00010011', huff(7) )
CALL sub_init_huff(0, 4,  '00010000', huff(7) )
CALL sub_init_huff(0, 5, '000001010', huff(7) )
CALL sub_init_huff(1, 0,       '011', huff(7) )
CALL sub_init_huff(1, 1,      '0011', huff(7) )
CALL sub_init_huff(1, 2,    '000111', huff(7) )
CALL sub_init_huff(1, 3,   '0001010', huff(7) )
CALL sub_init_huff(1, 4,   '0000101', huff(7) )
CALL sub_init_huff(1, 5,  '00000011', huff(7) )
CALL sub_init_huff(2, 0,    '001011', huff(7) )
CALL sub_init_huff(2, 1,     '00100', huff(7) )
CALL sub_init_huff(2, 2,   '0001101', huff(7) )
CALL sub_init_huff(2, 3,  '00010001', huff(7) )
CALL sub_init_huff(2, 4,  '00001000', huff(7) )
CALL sub_init_huff(2, 5, '000000100', huff(7) )
CALL sub_init_huff(3, 0,   '0001100', huff(7) )
CALL sub_init_huff(3, 1,   '0001011', huff(7) )
CALL sub_init_huff(3, 2,  '00010010', huff(7) )
CALL sub_init_huff(3, 3, '000001111', huff(7) )
CALL sub_init_huff(3, 4, '000001011', huff(7) )
CALL sub_init_huff(3, 5, '000000010', huff(7) )
CALL sub_init_huff(4, 0,   '0000111', huff(7) )
CALL sub_init_huff(4, 1,   '0000110', huff(7) )
CALL sub_init_huff(4, 2,  '00001001', huff(7) )
CALL sub_init_huff(4, 3, '000001110', huff(7) )
CALL sub_init_huff(4, 4, '000000011', huff(7) )
CALL sub_init_huff(4, 5,'0000000001', huff(7) )
CALL sub_init_huff(5, 0,  '00000110', huff(7) )
CALL sub_init_huff(5, 1,  '00000100', huff(7) )
CALL sub_init_huff(5, 2, '000000101', huff(7) )
CALL sub_init_huff(5, 3,'0000000011', huff(7) )
CALL sub_init_huff(5, 4,'0000000010', huff(7) )
CALL sub_init_huff(5, 5,'0000000000', huff(7) )
!
huff(8)%nmax    = 5
huff(8)%linbits = 0
CALL alloc_huff( huff(8) )
CALL sub_init_huff(0, 0,          '11', huff(8) )
CALL sub_init_huff(0, 1,         '100', huff(8) )
CALL sub_init_huff(0, 2,      '000110', huff(8) )
CALL sub_init_huff(0, 3,    '00010010', huff(8) )
CALL sub_init_huff(0, 4,    '00001100', huff(8) )
CALL sub_init_huff(0, 5,   '000000101', huff(8) )
CALL sub_init_huff(1, 0,         '101', huff(8) )
CALL sub_init_huff(1, 1,          '01', huff(8) )
CALL sub_init_huff(1, 2,        '0010', huff(8) )
CALL sub_init_huff(1, 3,    '00010000', huff(8) )
CALL sub_init_huff(1, 4,    '00001001', huff(8) )
CALL sub_init_huff(1, 5,    '00000011', huff(8) )
CALL sub_init_huff(2, 0,      '000111', huff(8) )
CALL sub_init_huff(2, 1,        '0011', huff(8) )
CALL sub_init_huff(2, 2,      '000101', huff(8) )
CALL sub_init_huff(2, 3,    '00001110', huff(8) )
CALL sub_init_huff(2, 4,    '00000111', huff(8) )
CALL sub_init_huff(2, 5,   '000000011', huff(8) )
CALL sub_init_huff(3, 0,    '00010011', huff(8) )
CALL sub_init_huff(3, 1,    '00010001', huff(8) )
CALL sub_init_huff(3, 2,    '00001111', huff(8) )
CALL sub_init_huff(3, 3,   '000001101', huff(8) )
CALL sub_init_huff(3, 4,   '000001010', huff(8) )
CALL sub_init_huff(3, 5,  '0000000100', huff(8) )
CALL sub_init_huff(4, 0,    '00001101', huff(8) )
CALL sub_init_huff(4, 1,     '0000101', huff(8) )
CALL sub_init_huff(4, 2,    '00001000', huff(8) )
CALL sub_init_huff(4, 3,   '000001011', huff(8) )
CALL sub_init_huff(4, 4,  '0000000101', huff(8) )
CALL sub_init_huff(4, 5,  '0000000001', huff(8) )
CALL sub_init_huff(5, 0,   '000001100', huff(8) )
CALL sub_init_huff(5, 1,    '00000100', huff(8) )
CALL sub_init_huff(5, 2,   '000000100', huff(8) )
CALL sub_init_huff(5, 3,   '000000001', huff(8) )
CALL sub_init_huff(5, 4, '00000000001', huff(8) )
CALL sub_init_huff(5, 5, '00000000000', huff(8) )
!
huff(9)%nmax    = 5
huff(9)%linbits = 0
CALL alloc_huff( huff(9) )
CALL sub_init_huff(0, 0,       '111', huff(9) )
CALL sub_init_huff(0, 1,       '101', huff(9) )
CALL sub_init_huff(0, 2,     '01001', huff(9) )
CALL sub_init_huff(0, 3,    '001110', huff(9) )
CALL sub_init_huff(0, 4,  '00001111', huff(9) )
CALL sub_init_huff(0, 5, '000000111', huff(9) )
CALL sub_init_huff(1, 0,       '110', huff(9) )
CALL sub_init_huff(1, 1,       '100', huff(9) )
CALL sub_init_huff(1, 2,      '0101', huff(9) )
CALL sub_init_huff(1, 3,     '00101', huff(9) )
CALL sub_init_huff(1, 4,    '000110', huff(9) )
CALL sub_init_huff(1, 5,  '00000111', huff(9) )
CALL sub_init_huff(2, 0,      '0111', huff(9) )
CALL sub_init_huff(2, 1,      '0110', huff(9) )
CALL sub_init_huff(2, 2,     '01000', huff(9) )
CALL sub_init_huff(2, 3,    '001000', huff(9) )
CALL sub_init_huff(2, 4,   '0001000', huff(9) )
CALL sub_init_huff(2, 5,  '00000101', huff(9) )
CALL sub_init_huff(3, 0,    '001111', huff(9) )
CALL sub_init_huff(3, 1,     '00110', huff(9) )
CALL sub_init_huff(3, 2,    '001001', huff(9) )
CALL sub_init_huff(3, 3,   '0001010', huff(9) )
CALL sub_init_huff(3, 4,   '0000101', huff(9) )
CALL sub_init_huff(3, 5,  '00000001', huff(9) )
CALL sub_init_huff(4, 0,   '0001011', huff(9) )
CALL sub_init_huff(4, 1,    '000111', huff(9) )
CALL sub_init_huff(4, 2,   '0001001', huff(9) )
CALL sub_init_huff(4, 3,   '0000110', huff(9) )
CALL sub_init_huff(4, 4,  '00000100', huff(9) )
CALL sub_init_huff(4, 5, '000000001', huff(9) )
CALL sub_init_huff(5, 0,  '00001110', huff(9) )
CALL sub_init_huff(5, 1,   '0000100', huff(9) )
CALL sub_init_huff(5, 2,  '00000110', huff(9) )
CALL sub_init_huff(5, 3,  '00000010', huff(9) )
CALL sub_init_huff(5, 4, '000000110', huff(9) )
CALL sub_init_huff(5, 5, '000000000', huff(9) )
!
huff(10)%nmax    = 7
huff(10)%linbits = 0
CALL alloc_huff( huff(10) )
CALL sub_init_huff(0, 0,           '1', huff(10) )
CALL sub_init_huff(0, 1,         '010', huff(10) )
CALL sub_init_huff(0, 2,      '001010', huff(10) )
CALL sub_init_huff(0, 3,    '00010111', huff(10) )
CALL sub_init_huff(0, 4,   '000100011', huff(10) )
CALL sub_init_huff(0, 5,   '000011110', huff(10) )
CALL sub_init_huff(0, 6,   '000001100', huff(10) )
CALL sub_init_huff(0, 7,  '0000010001', huff(10) )
CALL sub_init_huff(1, 0,         '011', huff(10) )
CALL sub_init_huff(1, 1,        '0011', huff(10) )
CALL sub_init_huff(1, 2,      '001000', huff(10) )
CALL sub_init_huff(1, 3,     '0001100', huff(10) )
CALL sub_init_huff(1, 4,    '00010010', huff(10) )
CALL sub_init_huff(1, 5,   '000010101', huff(10) )
CALL sub_init_huff(1, 6,    '00001100', huff(10) )
CALL sub_init_huff(1, 7,    '00000111', huff(10) )
CALL sub_init_huff(2, 0,      '001011', huff(10) )
CALL sub_init_huff(2, 1,      '001001', huff(10) )
CALL sub_init_huff(2, 2,     '0001111', huff(10) )
CALL sub_init_huff(2, 3,    '00010101', huff(10) )
CALL sub_init_huff(2, 4,   '000100000', huff(10) )
CALL sub_init_huff(2, 5,  '0000101000', huff(10) )
CALL sub_init_huff(2, 6,   '000010011', huff(10) )
CALL sub_init_huff(2, 7,   '000000110', huff(10) )
CALL sub_init_huff(3, 0,     '0001110', huff(10) )
CALL sub_init_huff(3, 1,     '0001101', huff(10) )
CALL sub_init_huff(3, 2,    '00010110', huff(10) )
CALL sub_init_huff(3, 3,   '000100010', huff(10) )
CALL sub_init_huff(3, 4,  '0000101110', huff(10) )
CALL sub_init_huff(3, 5,  '0000010111', huff(10) )
CALL sub_init_huff(3, 6,   '000010010', huff(10) )
CALL sub_init_huff(3, 7,  '0000000111', huff(10) )
CALL sub_init_huff(4, 0,    '00010100', huff(10) )
CALL sub_init_huff(4, 1,    '00010011', huff(10) )
CALL sub_init_huff(4, 2,   '000100001', huff(10) )
CALL sub_init_huff(4, 3,  '0000101111', huff(10) )
CALL sub_init_huff(4, 4,  '0000011011', huff(10) )
CALL sub_init_huff(4, 5,  '0000010110', huff(10) )
CALL sub_init_huff(4, 6,  '0000001001', huff(10) )
CALL sub_init_huff(4, 7,  '0000000011', huff(10) )
CALL sub_init_huff(5, 0,   '000011111', huff(10) )
CALL sub_init_huff(5, 1,   '000010110', huff(10) )
CALL sub_init_huff(5, 2,  '0000101001', huff(10) )
CALL sub_init_huff(5, 3,  '0000011010', huff(10) )
CALL sub_init_huff(5, 4, '00000010101', huff(10) )
CALL sub_init_huff(5, 5, '00000010100', huff(10) )
CALL sub_init_huff(5, 6,  '0000000101', huff(10) )
CALL sub_init_huff(5, 7, '00000000011', huff(10) )
CALL sub_init_huff(6, 0,    '00001110', huff(10) )
CALL sub_init_huff(6, 1,    '00001101', huff(10) )
CALL sub_init_huff(6, 2,   '000001010', huff(10) )
CALL sub_init_huff(6, 3,  '0000001011', huff(10) )
CALL sub_init_huff(6, 4,  '0000010000', huff(10) )
CALL sub_init_huff(6, 5,  '0000000110', huff(10) )
CALL sub_init_huff(6, 6, '00000000101', huff(10) )
CALL sub_init_huff(6, 7, '00000000001', huff(10) )
CALL sub_init_huff(7, 0,   '000001001', huff(10) )
CALL sub_init_huff(7, 1,    '00001000', huff(10) )
CALL sub_init_huff(7, 2,   '000000111', huff(10) )
CALL sub_init_huff(7, 3,  '0000001000', huff(10) )
CALL sub_init_huff(7, 4,  '0000000100', huff(10) )
CALL sub_init_huff(7, 5, '00000000100', huff(10) )
CALL sub_init_huff(7, 6, '00000000010', huff(10) )
CALL sub_init_huff(7, 7, '00000000000', huff(10) )
!
huff(11)%nmax    = 7
huff(11)%linbits = 0
CALL alloc_huff( huff(11) )
CALL sub_init_huff(0, 0,          '11', huff(11) )
CALL sub_init_huff(0, 1,         '100', huff(11) )
CALL sub_init_huff(0, 2,       '01010', huff(11) )
CALL sub_init_huff(0, 3,     '0011000', huff(11) ) 
CALL sub_init_huff(0, 4,    '00100010', huff(11) )
CALL sub_init_huff(0, 5,   '000100001', huff(11) )
CALL sub_init_huff(0, 6,    '00010101', huff(11) )
CALL sub_init_huff(0, 7,   '000001111', huff(11) )
CALL sub_init_huff(1, 0,         '101', huff(11) )
CALL sub_init_huff(1, 1,         '011', huff(11) )
CALL sub_init_huff(1, 2,        '0100', huff(11) )
CALL sub_init_huff(1, 3,      '001010', huff(11) )
CALL sub_init_huff(1, 4,    '00100000', huff(11) )
CALL sub_init_huff(1, 5,    '00010001', huff(11) )
CALL sub_init_huff(1, 6,     '0001011', huff(11) )
CALL sub_init_huff(1, 7,    '00001010', huff(11) )
CALL sub_init_huff(2, 0,       '01011', huff(11) )
CALL sub_init_huff(2, 1,       '00111', huff(11) )
CALL sub_init_huff(2, 2,      '001101', huff(11) )
CALL sub_init_huff(2, 3,     '0010010', huff(11) )
CALL sub_init_huff(2, 4,    '00011110', huff(11) )
CALL sub_init_huff(2, 5,   '000011111', huff(11) )
CALL sub_init_huff(2, 6,    '00010100', huff(11) )
CALL sub_init_huff(2, 7,    '00000101', huff(11) )
CALL sub_init_huff(3, 0,     '0011001', huff(11) )
CALL sub_init_huff(3, 1,      '001011', huff(11) )
CALL sub_init_huff(3, 2,     '0010011', huff(11) )
CALL sub_init_huff(3, 3,   '000111011', huff(11) )
CALL sub_init_huff(3, 4,    '00011011', huff(11) )
CALL sub_init_huff(3, 5,  '0000010010', huff(11) )
CALL sub_init_huff(3, 6,    '00001100', huff(11) )
CALL sub_init_huff(3, 7,   '000000101', huff(11) )
CALL sub_init_huff(4, 0,    '00100011', huff(11) )
CALL sub_init_huff(4, 1,    '00100001', huff(11) )
CALL sub_init_huff(4, 2,    '00011111', huff(11) )
CALL sub_init_huff(4, 3,   '000111010', huff(11) )
CALL sub_init_huff(4, 4,   '000011110', huff(11) )
CALL sub_init_huff(4, 5,  '0000010000', huff(11) )
CALL sub_init_huff(4, 6,   '000000111', huff(11) )
CALL sub_init_huff(4, 7,  '0000000101', huff(11) )
CALL sub_init_huff(5, 0,    '00011100', huff(11) )
CALL sub_init_huff(5, 1,    '00011010', huff(11) )
CALL sub_init_huff(5, 2,   '000100000', huff(11) )
CALL sub_init_huff(5, 3,  '0000010011', huff(11) )
CALL sub_init_huff(5, 4,  '0000010001', huff(11) )
CALL sub_init_huff(5, 5, '00000001111', huff(11) )
CALL sub_init_huff(5, 6,  '0000001000', huff(11) )
CALL sub_init_huff(5, 7, '00000001110', huff(11) )
CALL sub_init_huff(6, 0,    '00001110', huff(11) )
CALL sub_init_huff(6, 1,     '0001100', huff(11) )
CALL sub_init_huff(6, 2,     '0001001', huff(11) )
CALL sub_init_huff(6, 3,    '00001101', huff(11) )
CALL sub_init_huff(6, 4,   '000001110', huff(11) )
CALL sub_init_huff(6, 5,  '0000001001', huff(11) )
CALL sub_init_huff(6, 6,  '0000000100', huff(11) )
CALL sub_init_huff(6, 7,  '0000000001', huff(11) )
CALL sub_init_huff(7, 0,    '00001011', huff(11) )
CALL sub_init_huff(7, 1,     '0000100', huff(11) )
CALL sub_init_huff(7, 2,    '00000110', huff(11) )
CALL sub_init_huff(7, 3,   '000000110', huff(11) )
CALL sub_init_huff(7, 4,  '0000000110', huff(11) )
CALL sub_init_huff(7, 5,  '0000000011', huff(11) )
CALL sub_init_huff(7, 6,  '0000000010', huff(11) )
CALL sub_init_huff(7, 7,  '0000000000', huff(11) )
!
huff(12)%nmax    = 7
huff(12)%linbits = 0
CALL alloc_huff( huff(12) )
CALL sub_init_huff(0, 0,       '1001', huff(12) )
CALL sub_init_huff(0, 1,        '110', huff(12) )
CALL sub_init_huff(0, 2,      '10000', huff(12) )
CALL sub_init_huff(0, 3,    '0100001', huff(12) )
CALL sub_init_huff(0, 4,   '00101001', huff(12) )
CALL sub_init_huff(0, 5,  '000100111', huff(12) )
CALL sub_init_huff(0, 6,  '000100110', huff(12) )
CALL sub_init_huff(0, 7,  '000011010', huff(12) )
CALL sub_init_huff(1, 0,        '111', huff(12) )
CALL sub_init_huff(1, 1,        '101', huff(12) )
CALL sub_init_huff(1, 2,       '0110', huff(12) )
CALL sub_init_huff(1, 3,      '01001', huff(12) )
CALL sub_init_huff(1, 4,    '0010111', huff(12) )
CALL sub_init_huff(1, 5,    '0010000', huff(12) )
CALL sub_init_huff(1, 6,   '00011010', huff(12) )
CALL sub_init_huff(1, 7,   '00001011', huff(12) )
CALL sub_init_huff(2, 0,      '10001', huff(12) )
CALL sub_init_huff(2, 1,       '0111', huff(12) )
CALL sub_init_huff(2, 2,      '01011', huff(12) )
CALL sub_init_huff(2, 3,     '001110', huff(12) )
CALL sub_init_huff(2, 4,    '0010101', huff(12) )
CALL sub_init_huff(2, 5,   '00011110', huff(12) )
CALL sub_init_huff(2, 6,    '0001010', huff(12) )
CALL sub_init_huff(2, 7,   '00000111', huff(12) )
CALL sub_init_huff(3, 0,     '010001', huff(12) )
CALL sub_init_huff(3, 1,      '01010', huff(12) )
CALL sub_init_huff(3, 2,     '001111', huff(12) )
CALL sub_init_huff(3, 3,     '001100', huff(12) )
CALL sub_init_huff(3, 4,    '0010010', huff(12) )
CALL sub_init_huff(3, 5,   '00011100', huff(12) )
CALL sub_init_huff(3, 6,   '00001110', huff(12) )
CALL sub_init_huff(3, 7,   '00000101', huff(12) )
CALL sub_init_huff(4, 0,    '0100000', huff(12) )
CALL sub_init_huff(4, 1,     '001101', huff(12) )
CALL sub_init_huff(4, 2,    '0010110', huff(12) )
CALL sub_init_huff(4, 3,    '0010011', huff(12) )
CALL sub_init_huff(4, 4,   '00010010', huff(12) )
CALL sub_init_huff(4, 5,   '00010000', huff(12) )
CALL sub_init_huff(4, 6,   '00001001', huff(12) )
CALL sub_init_huff(4, 7,  '000000101', huff(12) )
CALL sub_init_huff(5, 0,   '00101000', huff(12) )
CALL sub_init_huff(5, 1,    '0010001', huff(12) )
CALL sub_init_huff(5, 2,   '00011111', huff(12) )
CALL sub_init_huff(5, 3,   '00011101', huff(12) )
CALL sub_init_huff(5, 4,   '00010001', huff(12) )
CALL sub_init_huff(5, 5,  '000001101', huff(12) )
CALL sub_init_huff(5, 6,   '00000100', huff(12) )
CALL sub_init_huff(5, 7,  '000000010', huff(12) )
CALL sub_init_huff(6, 0,   '00011011', huff(12) )
CALL sub_init_huff(6, 1,    '0001100', huff(12) )
CALL sub_init_huff(6, 2,    '0001011', huff(12) )
CALL sub_init_huff(6, 3,   '00001111', huff(12) )
CALL sub_init_huff(6, 4,   '00001010', huff(12) )
CALL sub_init_huff(6, 5,  '000000111', huff(12) )
CALL sub_init_huff(6, 6,  '000000100', huff(12) )
CALL sub_init_huff(6, 7, '0000000001', huff(12) )
CALL sub_init_huff(7, 0,  '000011011', huff(12) )
CALL sub_init_huff(7, 1,   '00001100', huff(12) )
CALL sub_init_huff(7, 2,   '00001000', huff(12) )
CALL sub_init_huff(7, 3,  '000001100', huff(12) )
CALL sub_init_huff(7, 4,  '000000110', huff(12) )
CALL sub_init_huff(7, 5,  '000000011', huff(12) )
CALL sub_init_huff(7, 6,  '000000001', huff(12) )
CALL sub_init_huff(7, 7, '0000000000', huff(12) )
!
huff(13)%nmax    = 15
huff(13)%linbits = 0
CALL alloc_huff( huff(13) )
CALL sub_init_huff(0,  0,             '1', huff(13) )
CALL sub_init_huff(0,  1,          '0101', huff(13) )
CALL sub_init_huff(0,  2,        '001110', huff(13) )
CALL sub_init_huff(0,  3,       '0010101', huff(13) )
CALL sub_init_huff(0,  4,      '00100010', huff(13) )
CALL sub_init_huff(0,  5,     '000110011', huff(13) )
CALL sub_init_huff(0,  6,     '000101110', huff(13) )
CALL sub_init_huff(0,  7,    '0001000111', huff(13) )
CALL sub_init_huff(0,  8,     '000101010', huff(13) )
CALL sub_init_huff(0,  9,    '0000110100', huff(13) )
CALL sub_init_huff(0, 10,   '00001000100', huff(13) )
CALL sub_init_huff(0, 11,   '00000110100', huff(13) )
CALL sub_init_huff(0, 12,  '000001000011', huff(13) )
CALL sub_init_huff(0, 13,  '000000101100', huff(13) )
CALL sub_init_huff(0, 14, '0000000101011', huff(13) )
CALL sub_init_huff(0, 15, '0000000010011', huff(13) )
!
CALL sub_init_huff(1,  0,          '011', huff(13) )
CALL sub_init_huff(1,  1,         '0100', huff(13) )
CALL sub_init_huff(1,  2,       '001100', huff(13) )
CALL sub_init_huff(1,  3,      '0010011', huff(13) )
CALL sub_init_huff(1,  4,     '00011111', huff(13) )
CALL sub_init_huff(1,  5,     '00011010', huff(13) )
CALL sub_init_huff(1,  6,    '000101100', huff(13) )
CALL sub_init_huff(1,  7,    '000100001', huff(13) )
CALL sub_init_huff(1,  8,    '000011111', huff(13) )
CALL sub_init_huff(1,  9,    '000011000', huff(13) )
CALL sub_init_huff(1, 10,   '0000100000', huff(13) )
CALL sub_init_huff(1, 11,   '0000011000', huff(13) )
CALL sub_init_huff(1, 12,  '00000011111', huff(13) )
CALL sub_init_huff(1, 13, '000000100011', huff(13) )
CALL sub_init_huff(1, 14, '000000010110', huff(13) )
CALL sub_init_huff(1, 15, '000000001110', huff(13) )
!
CALL sub_init_huff(2,  0,        '001111', huff(13) )
CALL sub_init_huff(2,  1,        '001101', huff(13) )
CALL sub_init_huff(2,  2,       '0010111', huff(13) )
CALL sub_init_huff(2,  3,      '00100100', huff(13) )
CALL sub_init_huff(2,  4,     '000111011', huff(13) )
CALL sub_init_huff(2,  5,     '000110001', huff(13) )
CALL sub_init_huff(2,  6,    '0001001101', huff(13) )
CALL sub_init_huff(2,  7,    '0001000001', huff(13) )
CALL sub_init_huff(2,  8,     '000011101', huff(13) )
CALL sub_init_huff(2,  9,    '0000101000', huff(13) )
CALL sub_init_huff(2, 10,    '0000011110', huff(13) )
CALL sub_init_huff(2, 11,   '00000101000', huff(13) )
CALL sub_init_huff(2, 12,   '00000011011', huff(13) )
CALL sub_init_huff(2, 13,  '000000100001', huff(13) )
CALL sub_init_huff(2, 14, '0000000101010', huff(13) )
CALL sub_init_huff(2, 15, '0000000010000', huff(13) )
!
CALL sub_init_huff(3,  0,       '0010110', huff(13) )
CALL sub_init_huff(3,  1,       '0010100', huff(13) )
CALL sub_init_huff(3,  2,      '00100101', huff(13) )
CALL sub_init_huff(3,  3,     '000111101', huff(13) )
CALL sub_init_huff(3,  4,     '000111000', huff(13) )
CALL sub_init_huff(3,  5,    '0001001111', huff(13) )
CALL sub_init_huff(3,  6,    '0001001001', huff(13) )
CALL sub_init_huff(3,  7,    '0001000000', huff(13) )
CALL sub_init_huff(3,  8,    '0000101011', huff(13) )
CALL sub_init_huff(3,  9,   '00001001100', huff(13) )
CALL sub_init_huff(3, 10,   '00000111000', huff(13) )
CALL sub_init_huff(3, 11,   '00000100101', huff(13) )
CALL sub_init_huff(3, 12,   '00000011010', huff(13) )
CALL sub_init_huff(3, 13,  '000000011111', huff(13) )
CALL sub_init_huff(3, 14, '0000000011001', huff(13) )
CALL sub_init_huff(3, 15, '0000000001110', huff(13) )
!
CALL sub_init_huff(4,  0,       '00100011', huff(13) )
CALL sub_init_huff(4,  1,        '0010000', huff(13) )
CALL sub_init_huff(4,  2,      '000111100', huff(13) )
CALL sub_init_huff(4,  3,      '000111001', huff(13) )
CALL sub_init_huff(4,  4,     '0001100001', huff(13) )
CALL sub_init_huff(4,  5,     '0001001011', huff(13) )
CALL sub_init_huff(4,  6,    '00001110010', huff(13) )
CALL sub_init_huff(4,  7,    '00001011011', huff(13) )
CALL sub_init_huff(4,  8,     '0000110110', huff(13) )
CALL sub_init_huff(4,  9,    '00001001001', huff(13) )
CALL sub_init_huff(4, 10,    '00000110111', huff(13) )
CALL sub_init_huff(4, 11,   '000000101001', huff(13) )
CALL sub_init_huff(4, 12,   '000000110000', huff(13) )
CALL sub_init_huff(4, 13,  '0000000110101', huff(13) )
CALL sub_init_huff(4, 14,  '0000000010111', huff(13) )
CALL sub_init_huff(4, 15, '00000000011000', huff(13) )
!
CALL sub_init_huff(5,  0,      '000111010', huff(13) )
CALL sub_init_huff(5,  1,       '00011011', huff(13) )
CALL sub_init_huff(5,  2,      '000110010', huff(13) )
CALL sub_init_huff(5,  3,     '0001100000', huff(13) )
CALL sub_init_huff(5,  4,     '0001001100', huff(13) )
CALL sub_init_huff(5,  5,     '0001000110', huff(13) )
CALL sub_init_huff(5,  6,    '00001011101', huff(13) )
CALL sub_init_huff(5,  7,    '00001010100', huff(13) )
CALL sub_init_huff(5,  8,    '00001001101', huff(13) )
CALL sub_init_huff(5,  9,    '00000111010', huff(13) )
CALL sub_init_huff(5, 10,   '000001001111', huff(13) )
CALL sub_init_huff(5, 11,    '00000011101', huff(13) )
CALL sub_init_huff(5, 12,  '0000001001010', huff(13) )
CALL sub_init_huff(5, 13,  '0000000110001', huff(13) )
CALL sub_init_huff(5, 14, '00000000101001', huff(13) )
CALL sub_init_huff(5, 15, '00000000010001', huff(13) )
!
CALL sub_init_huff(6,  0,      '000101111', huff(13) )
CALL sub_init_huff(6,  1,      '000101101', huff(13) )
CALL sub_init_huff(6,  2,     '0001001110', huff(13) )
CALL sub_init_huff(6,  3,     '0001001010', huff(13) )
CALL sub_init_huff(6,  4,    '00001110011', huff(13) )
CALL sub_init_huff(6,  5,    '00001011110', huff(13) )
CALL sub_init_huff(6,  6,    '00001011010', huff(13) )
CALL sub_init_huff(6,  7,    '00001001111', huff(13) )
CALL sub_init_huff(6,  8,    '00001000101', huff(13) )
CALL sub_init_huff(6,  9,   '000001010011', huff(13) )
CALL sub_init_huff(6, 10,   '000001000111', huff(13) )
CALL sub_init_huff(6, 11,   '000000110010', huff(13) )
CALL sub_init_huff(6, 12,  '0000000111011', huff(13) )
CALL sub_init_huff(6, 13,  '0000000100110', huff(13) )
CALL sub_init_huff(6, 14, '00000000100100', huff(13) )
CALL sub_init_huff(6, 15, '00000000001111', huff(13) )
!
CALL sub_init_huff(7,  0,       '0001001000', huff(13) )
CALL sub_init_huff(7,  1,        '000100010', huff(13) )
CALL sub_init_huff(7,  2,       '0000111000', huff(13) )
CALL sub_init_huff(7,  3,      '00001011111', huff(13) )
CALL sub_init_huff(7,  4,      '00001011100', huff(13) )
CALL sub_init_huff(7,  5,      '00001010101', huff(13) )
CALL sub_init_huff(7,  6,     '000001011011', huff(13) )
CALL sub_init_huff(7,  7,     '000001011010', huff(13) )
CALL sub_init_huff(7,  8,     '000001010110', huff(13) )
CALL sub_init_huff(7,  9,     '000001001001', huff(13) )
CALL sub_init_huff(7, 10,    '0000001001101', huff(13) )
CALL sub_init_huff(7, 11,    '0000001000001', huff(13) )
CALL sub_init_huff(7, 12,    '0000000110011', huff(13) )
CALL sub_init_huff(7, 13,   '00000000101100', huff(13) )
CALL sub_init_huff(7, 14, '0000000000101011', huff(13) )
CALL sub_init_huff(7, 15, '0000000000101010', huff(13) )
!
CALL sub_init_huff(8,  0,       '000101011', huff(13) )
CALL sub_init_huff(8,  1,        '00010100', huff(13) )
CALL sub_init_huff(8,  2,       '000011110', huff(13) )
CALL sub_init_huff(8,  3,      '0000101100', huff(13) )
CALL sub_init_huff(8,  4,      '0000110111', huff(13) )
CALL sub_init_huff(8,  5,     '00001001110', huff(13) )
CALL sub_init_huff(8,  6,     '00001001000', huff(13) )
CALL sub_init_huff(8,  7,    '000001010111', huff(13) )
CALL sub_init_huff(8,  8,    '000001001110', huff(13) )
CALL sub_init_huff(8,  9,    '000000111101', huff(13) )
CALL sub_init_huff(8, 10,    '000000101110', huff(13) )
CALL sub_init_huff(8, 11,   '0000000110110', huff(13) )
CALL sub_init_huff(8, 12,   '0000000100101', huff(13) )
CALL sub_init_huff(8, 13,  '00000000011110', huff(13) )
CALL sub_init_huff(8, 14, '000000000010100', huff(13) )
CALL sub_init_huff(8, 15, '000000000010000', huff(13) )
!
CALL sub_init_huff(9,  0,       '0000110101', huff(13) )
CALL sub_init_huff(9,  1,        '000011001', huff(13) )
CALL sub_init_huff(9,  2,       '0000101001', huff(13) )
CALL sub_init_huff(9,  3,       '0000100101', huff(13) )
CALL sub_init_huff(9,  4,      '00000101100', huff(13) )
CALL sub_init_huff(9,  5,      '00000111011', huff(13) )
CALL sub_init_huff(9,  6,      '00000110110', huff(13) )
CALL sub_init_huff(9,  7,    '0000001010001', huff(13) )
CALL sub_init_huff(9,  8,     '000001000010', huff(13) )
CALL sub_init_huff(9,  9,    '0000001001100', huff(13) )
CALL sub_init_huff(9, 10,    '0000000111001', huff(13) )
CALL sub_init_huff(9, 11,   '00000000110110', huff(13) )
CALL sub_init_huff(9, 12,   '00000000100101', huff(13) )
CALL sub_init_huff(9, 13,   '00000000010010', huff(13) )
CALL sub_init_huff(9, 14, '0000000000100111', huff(13) )
CALL sub_init_huff(9, 15,  '000000000001011', huff(13) )
!
CALL sub_init_huff(10,  0,        '0000100011', huff(13) )
CALL sub_init_huff(10,  1,        '0000100001', huff(13) )
CALL sub_init_huff(10,  2,        '0000011111', huff(13) )
CALL sub_init_huff(10,  3,       '00000111001', huff(13) )
CALL sub_init_huff(10,  4,       '00000101010', huff(13) )
CALL sub_init_huff(10,  5,      '000001010010', huff(13) )
CALL sub_init_huff(10,  6,      '000001001000', huff(13) )
CALL sub_init_huff(10,  7,     '0000001010000', huff(13) )
CALL sub_init_huff(10,  8,      '000000101111', huff(13) )
CALL sub_init_huff(10,  9,     '0000000111010', huff(13) )
CALL sub_init_huff(10, 10,    '00000000110111', huff(13) )
CALL sub_init_huff(10, 11,     '0000000010101', huff(13) )
CALL sub_init_huff(10, 12,    '00000000010110', huff(13) )
CALL sub_init_huff(10, 13,   '000000000011010', huff(13) )
CALL sub_init_huff(10, 14,  '0000000000100110', huff(13) )
CALL sub_init_huff(10, 15, '00000000000010110', huff(13) )
!
CALL sub_init_huff(11,  0,      '00000110101', huff(13) )
CALL sub_init_huff(11,  1,       '0000011001', huff(13) )
CALL sub_init_huff(11,  2,       '0000010111', huff(13) )
CALL sub_init_huff(11,  3,      '00000100110', huff(13) )
CALL sub_init_huff(11,  4,     '000001000110', huff(13) )
CALL sub_init_huff(11,  5,     '000000111100', huff(13) )
CALL sub_init_huff(11,  6,     '000000110011', huff(13) )
CALL sub_init_huff(11,  7,     '000000100100', huff(13) )
CALL sub_init_huff(11,  8,    '0000000110111', huff(13) )
CALL sub_init_huff(11,  9,    '0000000011010', huff(13) )
CALL sub_init_huff(11, 10,    '0000000100010', huff(13) )
CALL sub_init_huff(11, 11,   '00000000010111', huff(13) )
CALL sub_init_huff(11, 12,  '000000000011011', huff(13) )
CALL sub_init_huff(11, 13,  '000000000001110', huff(13) )
CALL sub_init_huff(11, 14,  '000000000001001', huff(13) )
CALL sub_init_huff(11, 15, '0000000000000111', huff(13) )
!
CALL sub_init_huff(12,  0,      '00000100010', huff(13) )
CALL sub_init_huff(12,  1,      '00000100000', huff(13) )
CALL sub_init_huff(12,  2,      '00000011100', huff(13) )
CALL sub_init_huff(12,  3,     '000000100111', huff(13) )
CALL sub_init_huff(12,  4,     '000000110001', huff(13) )
CALL sub_init_huff(12,  5,    '0000001001011', huff(13) )
CALL sub_init_huff(12,  6,     '000000011110', huff(13) )
CALL sub_init_huff(12,  7,    '0000000110100', huff(13) )
CALL sub_init_huff(12,  8,   '00000000110000', huff(13) )
CALL sub_init_huff(12,  9,   '00000000101000', huff(13) )
CALL sub_init_huff(12, 10,  '000000000110100', huff(13) )
CALL sub_init_huff(12, 11,  '000000000011100', huff(13) )
CALL sub_init_huff(12, 12,  '000000000010010', huff(13) )
CALL sub_init_huff(12, 13, '0000000000010001', huff(13) )
CALL sub_init_huff(12, 14, '0000000000001001', huff(13) )
CALL sub_init_huff(12, 15, '0000000000000101', huff(13) )
!
CALL sub_init_huff(13,  0,     '000000101101', huff(13) )
CALL sub_init_huff(13,  1,      '00000010101', huff(13) )
CALL sub_init_huff(13,  2,     '000000100010', huff(13) )
CALL sub_init_huff(13,  3,    '0000001000000', huff(13) )
CALL sub_init_huff(13,  4,    '0000000111000', huff(13) )
CALL sub_init_huff(13,  5,    '0000000110010', huff(13) )
CALL sub_init_huff(13,  6,   '00000000110001', huff(13) )
CALL sub_init_huff(13,  7,   '00000000101101', huff(13) )
CALL sub_init_huff(13,  8,   '00000000011111', huff(13) )
CALL sub_init_huff(13,  9,   '00000000010011', huff(13) )
CALL sub_init_huff(13, 10,   '00000000001100', huff(13) )
CALL sub_init_huff(13, 11,  '000000000001111', huff(13) )
CALL sub_init_huff(13, 12, '0000000000001010', huff(13) )
CALL sub_init_huff(13, 13,  '000000000000111', huff(13) )
CALL sub_init_huff(13, 14, '0000000000000110', huff(13) )
CALL sub_init_huff(13, 15, '0000000000000011', huff(13) )
!
CALL sub_init_huff(14,  0,     '0000000110000', huff(13) )
CALL sub_init_huff(14,  1,      '000000010111', huff(13) )
CALL sub_init_huff(14,  2,      '000000010100', huff(13) )
CALL sub_init_huff(14,  3,     '0000000100111', huff(13) )
CALL sub_init_huff(14,  4,     '0000000100100', huff(13) )
CALL sub_init_huff(14,  5,     '0000000100011', huff(13) )
CALL sub_init_huff(14,  6,   '000000000110101', huff(13) )
CALL sub_init_huff(14,  7,    '00000000010101', huff(13) )
CALL sub_init_huff(14,  8,    '00000000010000', huff(13) )
CALL sub_init_huff(14,  9, '00000000000010111', huff(13) )
CALL sub_init_huff(14, 10,   '000000000001101', huff(13) )
CALL sub_init_huff(14, 11,   '000000000001010', huff(13) )
CALL sub_init_huff(14, 12,   '000000000000110', huff(13) )
CALL sub_init_huff(14, 13, '00000000000000001', huff(13) )
CALL sub_init_huff(14, 14,  '0000000000000100', huff(13) )
CALL sub_init_huff(14, 15,  '0000000000000010', huff(13) )
!
CALL sub_init_huff(15,  0,        '000000010000', huff(13) )
CALL sub_init_huff(15,  1,        '000000001111', huff(13) )
CALL sub_init_huff(15,  2,       '0000000010001', huff(13) )
CALL sub_init_huff(15,  3,      '00000000011011', huff(13) )
CALL sub_init_huff(15,  4,      '00000000011001', huff(13) )
CALL sub_init_huff(15,  5,      '00000000010100', huff(13) )
CALL sub_init_huff(15,  6,     '000000000011101', huff(13) )
CALL sub_init_huff(15,  7,      '00000000001011', huff(13) )
CALL sub_init_huff(15,  8,     '000000000010001', huff(13) )
CALL sub_init_huff(15,  9,     '000000000001100', huff(13) )
CALL sub_init_huff(15, 10,    '0000000000010000', huff(13) )
CALL sub_init_huff(15, 11,    '0000000000001000', huff(13) )
CALL sub_init_huff(15, 12, '0000000000000000001', huff(13) )
CALL sub_init_huff(15, 13,  '000000000000000001', huff(13) )
CALL sub_init_huff(15, 14, '0000000000000000000', huff(13) )
CALL sub_init_huff(15, 15,    '0000000000000001', huff(13) )
!
huff(14)%nmax    = -1
huff(14)%linbits =  0
huff(14)%linmax  = -1
!
huff(15)%nmax    = 15
huff(15)%linbits = 0
CALL alloc_huff( huff(15) )
CALL sub_init_huff( 0,  0,           '111', huff(15) )
CALL sub_init_huff( 0,  1,          '1100', huff(15) )
CALL sub_init_huff( 0,  2,         '10010', huff(15) )
CALL sub_init_huff( 0,  3,       '0110101', huff(15) )
CALL sub_init_huff( 0,  4,       '0101111', huff(15) )
CALL sub_init_huff( 0,  5,      '01001100', huff(15) )
CALL sub_init_huff( 0,  6,     '001111100', huff(15) )
CALL sub_init_huff( 0,  7,     '001101100', huff(15) )
CALL sub_init_huff( 0,  8,     '001011001', huff(15) )
CALL sub_init_huff( 0,  9,    '0001111011', huff(15) )
CALL sub_init_huff( 0, 10,    '0001101100', huff(15) )
CALL sub_init_huff( 0, 11,   '00001110111', huff(15) )
CALL sub_init_huff( 0, 12,   '00001101011', huff(15) )
CALL sub_init_huff( 0, 13,   '00001010001', huff(15) )
CALL sub_init_huff( 0, 14,  '000001111010', huff(15) )
CALL sub_init_huff( 0, 15, '0000000111111', huff(15) )
!
CALL sub_init_huff( 1,  0,        '1101', huff(15) )
CALL sub_init_huff( 1,  1,         '101', huff(15) )
CALL sub_init_huff( 1,  2,       '10000', huff(15) )
CALL sub_init_huff( 1,  3,      '011011', huff(15) )
CALL sub_init_huff( 1,  4,     '0101110', huff(15) )
CALL sub_init_huff( 1,  5,     '0100100', huff(15) )
CALL sub_init_huff( 1,  6,    '00111101', huff(15) )
CALL sub_init_huff( 1,  7,    '00110011', huff(15) )
CALL sub_init_huff( 1,  8,    '00101010', huff(15) )
CALL sub_init_huff( 1,  9,   '001000110', huff(15) )
CALL sub_init_huff( 1, 10,   '000110100', huff(15) )
CALL sub_init_huff( 1, 11,  '0001010011', huff(15) )
CALL sub_init_huff( 1, 12,  '0001000001', huff(15) )
CALL sub_init_huff( 1, 13,  '0000101001', huff(15) )
CALL sub_init_huff( 1, 14, '00000111011', huff(15) )
CALL sub_init_huff( 1, 15, '00000100100', huff(15) )
!
CALL sub_init_huff( 2,  0,       '10011', huff(15) )
CALL sub_init_huff( 2,  1,       '10001', huff(15) )
CALL sub_init_huff( 2,  2,       '01111', huff(15) )
CALL sub_init_huff( 2,  3,      '011000', huff(15) )
CALL sub_init_huff( 2,  4,     '0101001', huff(15) )
CALL sub_init_huff( 2,  5,     '0100010', huff(15) )
CALL sub_init_huff( 2,  6,    '00111011', huff(15) )
CALL sub_init_huff( 2,  7,    '00110000', huff(15) )
CALL sub_init_huff( 2,  8,    '00101000', huff(15) )
CALL sub_init_huff( 2,  9,   '001000000', huff(15) )
CALL sub_init_huff( 2, 10,   '000110010', huff(15) )
CALL sub_init_huff( 2, 11,  '0001001110', huff(15) )
CALL sub_init_huff( 2, 12,  '0000111110', huff(15) )
CALL sub_init_huff( 2, 13, '00001010000', huff(15) )
CALL sub_init_huff( 2, 14, '00000111000', huff(15) )
CALL sub_init_huff( 2, 15, '00000100001', huff(15) )
!
CALL sub_init_huff( 3,  0,      '011101', huff(15) )
CALL sub_init_huff( 3,  1,      '011100', huff(15) )
CALL sub_init_huff( 3,  2,      '011001', huff(15) )
CALL sub_init_huff( 3,  3,     '0101011', huff(15) )
CALL sub_init_huff( 3,  4,     '0100111', huff(15) )
CALL sub_init_huff( 3,  5,    '00111111', huff(15) )
CALL sub_init_huff( 3,  6,    '00110111', huff(15) )
CALL sub_init_huff( 3,  7,   '001011101', huff(15) )
CALL sub_init_huff( 3,  8,   '001001100', huff(15) )
CALL sub_init_huff( 3,  9,   '000111011', huff(15) )
CALL sub_init_huff( 3, 10,  '0001011101', huff(15) )
CALL sub_init_huff( 3, 11,  '0001001000', huff(15) )
CALL sub_init_huff( 3, 12,  '0000110110', huff(15) )
CALL sub_init_huff( 3, 13, '00001001011', huff(15) )
CALL sub_init_huff( 3, 14, '00000110010', huff(15) )
CALL sub_init_huff( 3, 15, '00000011101', huff(15) )
!
CALL sub_init_huff( 4,  0,     '0110100', huff(15) )
CALL sub_init_huff( 4,  1,      '010110', huff(15) )
CALL sub_init_huff( 4,  2,     '0101010', huff(15) )
CALL sub_init_huff( 4,  3,     '0101000', huff(15) )
CALL sub_init_huff( 4,  4,    '01000011', huff(15) )
CALL sub_init_huff( 4,  5,    '00111001', huff(15) )
CALL sub_init_huff( 4,  6,   '001011111', huff(15) )
CALL sub_init_huff( 4,  7,   '001001111', huff(15) )
CALL sub_init_huff( 4,  8,   '001001000', huff(15) )
CALL sub_init_huff( 4,  9,   '000111001', huff(15) )
CALL sub_init_huff( 4, 10,  '0001011001', huff(15) )
CALL sub_init_huff( 4, 11,  '0001000101', huff(15) )
CALL sub_init_huff( 4, 12,  '0000110001', huff(15) )
CALL sub_init_huff( 4, 13, '00001000010', huff(15) )
CALL sub_init_huff( 4, 14, '00000101110', huff(15) )
CALL sub_init_huff( 4, 15, '00000011011', huff(15) )
!
CALL sub_init_huff( 5,  0,     '01001101', huff(15) )
CALL sub_init_huff( 5,  1,      '0100101', huff(15) )
CALL sub_init_huff( 5,  2,      '0100011', huff(15) )
CALL sub_init_huff( 5,  3,     '01000010', huff(15) )
CALL sub_init_huff( 5,  4,     '00111010', huff(15) )
CALL sub_init_huff( 5,  5,     '00110100', huff(15) )
CALL sub_init_huff( 5,  6,    '001011011', huff(15) )
CALL sub_init_huff( 5,  7,    '001001010', huff(15) )
CALL sub_init_huff( 5,  8,    '000111110', huff(15) )
CALL sub_init_huff( 5,  9,    '000110000', huff(15) )
CALL sub_init_huff( 5, 10,   '0001001111', huff(15) )
CALL sub_init_huff( 5, 11,   '0000111111', huff(15) )
CALL sub_init_huff( 5, 12,  '00001011010', huff(15) )
CALL sub_init_huff( 5, 13,  '00000111110', huff(15) )
CALL sub_init_huff( 5, 14,  '00000101000', huff(15) )
CALL sub_init_huff( 5, 15, '000000100110', huff(15) )
!
CALL sub_init_huff( 6,  0,    '001111101', huff(15) )
CALL sub_init_huff( 6,  1,      '0100000', huff(15) )
CALL sub_init_huff( 6,  2,     '00111100', huff(15) )
CALL sub_init_huff( 6,  3,     '00111000', huff(15) )
CALL sub_init_huff( 6,  4,     '00110010', huff(15) )
CALL sub_init_huff( 6,  5,    '001011100', huff(15) )
CALL sub_init_huff( 6,  6,    '001001110', huff(15) )
CALL sub_init_huff( 6,  7,    '001000001', huff(15) )
CALL sub_init_huff( 6,  8,    '000110111', huff(15) )
CALL sub_init_huff( 6,  9,   '0001010111', huff(15) )
CALL sub_init_huff( 6, 10,   '0001000111', huff(15) )
CALL sub_init_huff( 6, 11,   '0000110011', huff(15) )
CALL sub_init_huff( 6, 12,  '00001001001', huff(15) )
CALL sub_init_huff( 6, 13,  '00000110011', huff(15) )
CALL sub_init_huff( 6, 14, '000001000110', huff(15) )
CALL sub_init_huff( 6, 15, '000000011110', huff(15) )
!
CALL sub_init_huff( 7,  0,    '001101101', huff(15) )
CALL sub_init_huff( 7,  1,     '00110101', huff(15) )
CALL sub_init_huff( 7,  2,     '00110001', huff(15) )
CALL sub_init_huff( 7,  3,    '001011110', huff(15) )
CALL sub_init_huff( 7,  4,    '001011000', huff(15) )
CALL sub_init_huff( 7,  5,    '001001011', huff(15) )
CALL sub_init_huff( 7,  6,    '001000010', huff(15) )
CALL sub_init_huff( 7,  7,   '0001111010', huff(15) )
CALL sub_init_huff( 7,  8,   '0001011011', huff(15) )
CALL sub_init_huff( 7,  9,   '0001001001', huff(15) )
CALL sub_init_huff( 7, 10,   '0000111000', huff(15) )
CALL sub_init_huff( 7, 11,   '0000101010', huff(15) )
CALL sub_init_huff( 7, 12,  '00001000000', huff(15) )
CALL sub_init_huff( 7, 13,  '00000101100', huff(15) )
CALL sub_init_huff( 7, 14,  '00000010101', huff(15) )
CALL sub_init_huff( 7, 15, '000000011001', huff(15) )
!
CALL sub_init_huff( 8,  0,    '001011010', huff(15) )
CALL sub_init_huff( 8,  1,     '00101011', huff(15) )
CALL sub_init_huff( 8,  2,     '00101001', huff(15) )
CALL sub_init_huff( 8,  3,    '001001101', huff(15) )
CALL sub_init_huff( 8,  4,    '001001001', huff(15) )
CALL sub_init_huff( 8,  5,    '000111111', huff(15) )
CALL sub_init_huff( 8,  6,    '000111000', huff(15) )
CALL sub_init_huff( 8,  7,   '0001011100', huff(15) )
CALL sub_init_huff( 8,  8,   '0001001101', huff(15) )
CALL sub_init_huff( 8,  9,   '0001000010', huff(15) )
CALL sub_init_huff( 8, 10,   '0000101111', huff(15) )
CALL sub_init_huff( 8, 11,  '00001000011', huff(15) )
CALL sub_init_huff( 8, 12,  '00000110000', huff(15) )
CALL sub_init_huff( 8, 13, '000000110101', huff(15) )
CALL sub_init_huff( 8, 14, '000000100100', huff(15) )
CALL sub_init_huff( 8, 15, '000000010100', huff(15) )
!
CALL sub_init_huff( 9,  0,    '001000111', huff(15) )
CALL sub_init_huff( 9,  1,     '00100010', huff(15) )
CALL sub_init_huff( 9,  2,    '001000011', huff(15) )
CALL sub_init_huff( 9,  3,    '000111100', huff(15) )
CALL sub_init_huff( 9,  4,    '000111010', huff(15) )
CALL sub_init_huff( 9,  5,    '000110001', huff(15) )
CALL sub_init_huff( 9,  6,   '0001011000', huff(15) )
CALL sub_init_huff( 9,  7,   '0001001100', huff(15) )
CALL sub_init_huff( 9,  8,   '0001000011', huff(15) )
CALL sub_init_huff( 9,  9,  '00001101010', huff(15) )
CALL sub_init_huff( 9, 10,  '00001000111', huff(15) )
CALL sub_init_huff( 9, 11,  '00000110110', huff(15) )
CALL sub_init_huff( 9, 12,  '00000100110', huff(15) )
CALL sub_init_huff( 9, 13, '000000100111', huff(15) )
CALL sub_init_huff( 9, 14, '000000010111', huff(15) )
CALL sub_init_huff( 9, 15, '000000001111', huff(15) )
!
CALL sub_init_huff(10,  0,    '0001101101', huff(15) )
CALL sub_init_huff(10,  1,     '000110101', huff(15) )
CALL sub_init_huff(10,  2,     '000110011', huff(15) )
CALL sub_init_huff(10,  3,     '000101111', huff(15) )
CALL sub_init_huff(10,  4,    '0001011010', huff(15) )
CALL sub_init_huff(10,  5,    '0001010010', huff(15) )
CALL sub_init_huff(10,  6,    '0000111010', huff(15) )
CALL sub_init_huff(10,  7,    '0000111001', huff(15) )
CALL sub_init_huff(10,  8,    '0000110000', huff(15) )
CALL sub_init_huff(10,  9,   '00001001000', huff(15) )
CALL sub_init_huff(10, 10,   '00000111001', huff(15) )
CALL sub_init_huff(10, 11,   '00000101001', huff(15) )
CALL sub_init_huff(10, 12,   '00000010111', huff(15) )
CALL sub_init_huff(10, 13,  '000000011011', huff(15) )
CALL sub_init_huff(10, 14, '0000000111110', huff(15) )
CALL sub_init_huff(10, 15,  '000000001001', huff(15) )
!
CALL sub_init_huff(11,  0,    '0001010110', huff(15) )
CALL sub_init_huff(11,  1,     '000101010', huff(15) )
CALL sub_init_huff(11,  2,     '000101000', huff(15) )
CALL sub_init_huff(11,  3,     '000100101', huff(15) )
CALL sub_init_huff(11,  4,    '0001000110', huff(15) )
CALL sub_init_huff(11,  5,    '0001000000', huff(15) )
CALL sub_init_huff(11,  6,    '0000110100', huff(15) )
CALL sub_init_huff(11,  7,    '0000101011', huff(15) )
CALL sub_init_huff(11,  8,   '00001000110', huff(15) )
CALL sub_init_huff(11,  9,   '00000110111', huff(15) )
CALL sub_init_huff(11, 10,   '00000101010', huff(15) )
CALL sub_init_huff(11, 11,   '00000011001', huff(15) )
CALL sub_init_huff(11, 12,  '000000011101', huff(15) )
CALL sub_init_huff(11, 13,  '000000010010', huff(15) )
CALL sub_init_huff(11, 14,  '000000001011', huff(15) )
CALL sub_init_huff(11, 15, '0000000001011', huff(15) )
!
CALL sub_init_huff(12,  0,   '00001110110', huff(15) )
CALL sub_init_huff(12,  1,    '0001000100', huff(15) )
CALL sub_init_huff(12,  2,     '000011110', huff(15) )
CALL sub_init_huff(12,  3,    '0000110111', huff(15) )
CALL sub_init_huff(12,  4,    '0000110010', huff(15) )
CALL sub_init_huff(12,  5,    '0000101110', huff(15) )
CALL sub_init_huff(12,  6,   '00001001010', huff(15) )
CALL sub_init_huff(12,  7,   '00001000001', huff(15) )
CALL sub_init_huff(12,  8,   '00000110001', huff(15) )
CALL sub_init_huff(12,  9,   '00000100111', huff(15) )
CALL sub_init_huff(12, 10,   '00000011000', huff(15) )
CALL sub_init_huff(12, 11,   '00000010000', huff(15) )
CALL sub_init_huff(12, 12,  '000000010110', huff(15) )
CALL sub_init_huff(12, 13,  '000000001101', huff(15) )
CALL sub_init_huff(12, 14, '0000000001110', huff(15) )
CALL sub_init_huff(12, 15, '0000000000111', huff(15) )
!
CALL sub_init_huff(13,  0,   '00001011011', huff(15) )
CALL sub_init_huff(13,  1,    '0000101100', huff(15) )
CALL sub_init_huff(13,  2,    '0000100111', huff(15) )
CALL sub_init_huff(13,  3,    '0000100110', huff(15) )
CALL sub_init_huff(13,  4,    '0000100010', huff(15) )
CALL sub_init_huff(13,  5,   '00000111111', huff(15) )
CALL sub_init_huff(13,  6,   '00000110100', huff(15) )
CALL sub_init_huff(13,  7,   '00000101101', huff(15) )
CALL sub_init_huff(13,  8,   '00000011111', huff(15) )
CALL sub_init_huff(13,  9,  '000000110100', huff(15) )
CALL sub_init_huff(13, 10,  '000000011100', huff(15) )
CALL sub_init_huff(13, 11,  '000000010011', huff(15) )
CALL sub_init_huff(13, 12,  '000000001110', huff(15) )
CALL sub_init_huff(13, 13,  '000000001000', huff(15) )
CALL sub_init_huff(13, 14, '0000000001001', huff(15) )
CALL sub_init_huff(13, 15, '0000000000011', huff(15) )
!
CALL sub_init_huff(14,  0,  '000001111011', huff(15) )
CALL sub_init_huff(14,  1,   '00000111100', huff(15) )
CALL sub_init_huff(14,  2,   '00000111010', huff(15) )
CALL sub_init_huff(14,  3,   '00000110101', huff(15) )
CALL sub_init_huff(14,  4,   '00000101111', huff(15) )
CALL sub_init_huff(14,  5,   '00000101011', huff(15) )
CALL sub_init_huff(14,  6,   '00000100000', huff(15) )
CALL sub_init_huff(14,  7,   '00000010110', huff(15) )
CALL sub_init_huff(14,  8,  '000000100101', huff(15) )
CALL sub_init_huff(14,  9,  '000000011000', huff(15) )
CALL sub_init_huff(14, 10,  '000000010001', huff(15) )
CALL sub_init_huff(14, 11,  '000000001100', huff(15) )
CALL sub_init_huff(14, 12, '0000000001111', huff(15) )
CALL sub_init_huff(14, 13, '0000000001010', huff(15) )
CALL sub_init_huff(14, 14,  '000000000010', huff(15) )
CALL sub_init_huff(14, 15, '0000000000001', huff(15) )
!
CALL sub_init_huff(15,  0,  '000001000111', huff(15) )
CALL sub_init_huff(15,  1,   '00000100101', huff(15) )
CALL sub_init_huff(15,  2,   '00000100010', huff(15) )
CALL sub_init_huff(15,  3,   '00000011110', huff(15) )
CALL sub_init_huff(15,  4,   '00000011100', huff(15) )
CALL sub_init_huff(15,  5,   '00000010100', huff(15) )
CALL sub_init_huff(15,  6,   '00000010001', huff(15) )
CALL sub_init_huff(15,  7,  '000000011010', huff(15) )
CALL sub_init_huff(15,  8,  '000000010101', huff(15) )
CALL sub_init_huff(15,  9,  '000000010000', huff(15) )
CALL sub_init_huff(15, 10,  '000000001010', huff(15) )
CALL sub_init_huff(15, 11,  '000000000110', huff(15) )
CALL sub_init_huff(15, 12, '0000000001000', huff(15) )
CALL sub_init_huff(15, 13, '0000000000110', huff(15) )
CALL sub_init_huff(15, 14, '0000000000010', huff(15) )
CALL sub_init_huff(15, 15, '0000000000000', huff(15) )
!
huff(16)%nmax    = 15
huff(16)%linbits = 1
CALL alloc_huff( huff(16) )
CALL sub_init_huff( 0,  0,             '1', huff(16) )
CALL sub_init_huff( 0,  1,          '0101', huff(16) )
CALL sub_init_huff( 0,  2,        '001110', huff(16) )
CALL sub_init_huff( 0,  3,      '00101100', huff(16) )
CALL sub_init_huff( 0,  4,     '001001010', huff(16) )
CALL sub_init_huff( 0,  5,     '000111111', huff(16) )
CALL sub_init_huff( 0,  6,    '0001101110', huff(16) )
CALL sub_init_huff( 0,  7,    '0001011101', huff(16) )
CALL sub_init_huff( 0,  8,   '00010101100', huff(16) )
CALL sub_init_huff( 0,  9,   '00010010101', huff(16) )
CALL sub_init_huff( 0, 10,   '00010001010', huff(16) )
CALL sub_init_huff( 0, 11,  '000011110010', huff(16) )
CALL sub_init_huff( 0, 12,  '000011100001', huff(16) )
CALL sub_init_huff( 0, 13,  '000011000011', huff(16) )
CALL sub_init_huff( 0, 14, '0000101111000', huff(16) )
CALL sub_init_huff( 0, 15,     '000010001', huff(16) )
!
CALL sub_init_huff( 1,  0,          '011', huff(16) )
CALL sub_init_huff( 1,  1,         '0100', huff(16) )
CALL sub_init_huff( 1,  2,       '001100', huff(16) )
CALL sub_init_huff( 1,  3,      '0010100', huff(16) )
CALL sub_init_huff( 1,  4,     '00100011', huff(16) )
CALL sub_init_huff( 1,  5,    '000111110', huff(16) )
CALL sub_init_huff( 1,  6,    '000110101', huff(16) )
CALL sub_init_huff( 1,  7,    '000101111', huff(16) )
CALL sub_init_huff( 1,  8,   '0001010011', huff(16) )
CALL sub_init_huff( 1,  9,   '0001001011', huff(16) )
CALL sub_init_huff( 1, 10,   '0001000100', huff(16) )
CALL sub_init_huff( 1, 11,  '00001110111', huff(16) )
CALL sub_init_huff( 1, 12, '000011001001', huff(16) )
CALL sub_init_huff( 1, 13,  '00001101011', huff(16) )
CALL sub_init_huff( 1, 14, '000011001111', huff(16) )
CALL sub_init_huff( 1, 15,     '00001001', huff(16) )
!
CALL sub_init_huff( 2,  0,       '001111', huff(16) )
CALL sub_init_huff( 2,  1,       '001101', huff(16) )
CALL sub_init_huff( 2,  2,      '0010111', huff(16) )
CALL sub_init_huff( 2,  3,     '00100110', huff(16) )
CALL sub_init_huff( 2,  4,    '001000011', huff(16) )
CALL sub_init_huff( 2,  5,    '000111010', huff(16) )
CALL sub_init_huff( 2,  6,   '0001100111', huff(16) )
CALL sub_init_huff( 2,  7,   '0001011010', huff(16) )
CALL sub_init_huff( 2,  8,  '00010100001', huff(16) )
CALL sub_init_huff( 2,  9,   '0001001000', huff(16) )
CALL sub_init_huff( 2, 10,  '00001111111', huff(16) )
CALL sub_init_huff( 2, 11,  '00001110101', huff(16) )
CALL sub_init_huff( 2, 12,  '00001101110', huff(16) )
CALL sub_init_huff( 2, 13, '000011010001', huff(16) )
CALL sub_init_huff( 2, 14, '000011001110', huff(16) )
CALL sub_init_huff( 2, 15,    '000010000', huff(16) )
!
CALL sub_init_huff( 3,  0,      '00101101', huff(16) )
CALL sub_init_huff( 3,  1,       '0010101', huff(16) )
CALL sub_init_huff( 3,  2,      '00100111', huff(16) )
CALL sub_init_huff( 3,  3,     '001000101', huff(16) )
CALL sub_init_huff( 3,  4,     '001000000', huff(16) )
CALL sub_init_huff( 3,  5,    '0001110010', huff(16) )
CALL sub_init_huff( 3,  6,    '0001100011', huff(16) )
CALL sub_init_huff( 3,  7,    '0001010111', huff(16) )
CALL sub_init_huff( 3,  8,   '00010011110', huff(16) )
CALL sub_init_huff( 3,  9,   '00010001100', huff(16) )
CALL sub_init_huff( 3, 10,  '000011111100', huff(16) )
CALL sub_init_huff( 3, 11,  '000011010100', huff(16) )
CALL sub_init_huff( 3, 12,  '000011000111', huff(16) )
CALL sub_init_huff( 3, 13, '0000110000011', huff(16) )
CALL sub_init_huff( 3, 14, '0000101101101', huff(16) )
CALL sub_init_huff( 3, 15,    '0000011010', huff(16) )
!
CALL sub_init_huff( 4,  0,     '001001011', huff(16) )
CALL sub_init_huff( 4,  1,      '00100100', huff(16) )
CALL sub_init_huff( 4,  2,     '001000100', huff(16) )
CALL sub_init_huff( 4,  3,     '001000001', huff(16) )
CALL sub_init_huff( 4,  4,    '0001110011', huff(16) )
CALL sub_init_huff( 4,  5,    '0001100101', huff(16) )
CALL sub_init_huff( 4,  6,   '00010110011', huff(16) )
CALL sub_init_huff( 4,  7,   '00010100100', huff(16) )
CALL sub_init_huff( 4,  8,   '00010011011', huff(16) )
CALL sub_init_huff( 4,  9,  '000100001000', huff(16) )
CALL sub_init_huff( 4, 10,  '000011110110', huff(16) )
CALL sub_init_huff( 4, 11,  '000011100010', huff(16) )
CALL sub_init_huff( 4, 12, '0000110001011', huff(16) )
CALL sub_init_huff( 4, 13, '0000101111110', huff(16) )
CALL sub_init_huff( 4, 14, '0000101101010', huff(16) )
CALL sub_init_huff( 4, 15,     '000001001', huff(16) )
!
CALL sub_init_huff( 5,  0,      '001000010', huff(16) )
CALL sub_init_huff( 5,  1,       '00011110', huff(16) )
CALL sub_init_huff( 5,  2,      '000111011', huff(16) )
CALL sub_init_huff( 5,  3,      '000111000', huff(16) )
CALL sub_init_huff( 5,  4,     '0001100110', huff(16) )
CALL sub_init_huff( 5,  5,    '00010111001', huff(16) )
CALL sub_init_huff( 5,  6,    '00010101101', huff(16) )
CALL sub_init_huff( 5,  7,   '000100001001', huff(16) )
CALL sub_init_huff( 5,  8,    '00010001110', huff(16) )
CALL sub_init_huff( 5,  9,   '000011111101', huff(16) )
CALL sub_init_huff( 5, 10,   '000011101000', huff(16) )
CALL sub_init_huff( 5, 11,  '0000110010000', huff(16) )
CALL sub_init_huff( 5, 12,  '0000110000100', huff(16) )
CALL sub_init_huff( 5, 13,  '0000101111010', huff(16) )
CALL sub_init_huff( 5, 14, '00000110111101', huff(16) )
CALL sub_init_huff( 5, 15,     '0000010000', huff(16) )
!
CALL sub_init_huff( 6,  0,     '0001101111', huff(16) )
CALL sub_init_huff( 6,  1,      '000110110', huff(16) )
CALL sub_init_huff( 6,  2,      '000110100', huff(16) )
CALL sub_init_huff( 6,  3,     '0001100100', huff(16) )
CALL sub_init_huff( 6,  4,    '00010111000', huff(16) )
CALL sub_init_huff( 6,  5,    '00010110010', huff(16) )
CALL sub_init_huff( 6,  6,    '00010100000', huff(16) )
CALL sub_init_huff( 6,  7,    '00010000101', huff(16) )
CALL sub_init_huff( 6,  8,   '000100000001', huff(16) )
CALL sub_init_huff( 6,  9,   '000011110100', huff(16) )
CALL sub_init_huff( 6, 10,   '000011100100', huff(16) )
CALL sub_init_huff( 6, 11,   '000011011001', huff(16) )
CALL sub_init_huff( 6, 12,  '0000110000001', huff(16) )
CALL sub_init_huff( 6, 13,  '0000101101110', huff(16) )
CALL sub_init_huff( 6, 14, '00001011001011', huff(16) )
CALL sub_init_huff( 6, 15,     '0000001010', huff(16) )
!
CALL sub_init_huff( 7,  0,      '0001100010', huff(16) )
CALL sub_init_huff( 7,  1,       '000110000', huff(16) )
CALL sub_init_huff( 7,  2,      '0001011011', huff(16) )
CALL sub_init_huff( 7,  3,      '0001011000', huff(16) )
CALL sub_init_huff( 7,  4,     '00010100101', huff(16) )
CALL sub_init_huff( 7,  5,     '00010011101', huff(16) )
CALL sub_init_huff( 7,  6,     '00010010100', huff(16) )
CALL sub_init_huff( 7,  7,    '000100000101', huff(16) )
CALL sub_init_huff( 7,  8,    '000011111000', huff(16) )
CALL sub_init_huff( 7,  9,   '0000110010111', huff(16) )
CALL sub_init_huff( 7, 10,   '0000110001101', huff(16) )
CALL sub_init_huff( 7, 11,   '0000101110100', huff(16) )
CALL sub_init_huff( 7, 12,   '0000101111100', huff(16) )
CALL sub_init_huff( 7, 13, '000001101111001', huff(16) )
CALL sub_init_huff( 7, 14, '000001101110100', huff(16) )
CALL sub_init_huff( 7, 15,      '0000001000', huff(16) )
!
CALL sub_init_huff( 8,  0,     '0001010101', huff(16) )
CALL sub_init_huff( 8,  1,     '0001010100', huff(16) )
CALL sub_init_huff( 8,  2,     '0001010001', huff(16) )
CALL sub_init_huff( 8,  3,    '00010011111', huff(16) )
CALL sub_init_huff( 8,  4,    '00010011100', huff(16) )
CALL sub_init_huff( 8,  5,    '00010001111', huff(16) )
CALL sub_init_huff( 8,  6,   '000100000100', huff(16) )
CALL sub_init_huff( 8,  7,   '000011111001', huff(16) )
CALL sub_init_huff( 8,  8,  '0000110101011', huff(16) )
CALL sub_init_huff( 8,  9,  '0000110010001', huff(16) )
CALL sub_init_huff( 8, 10,  '0000110001000', huff(16) )
CALL sub_init_huff( 8, 11,  '0000101111111', huff(16) )
CALL sub_init_huff( 8, 12, '00001011010111', huff(16) )
CALL sub_init_huff( 8, 13, '00001011001001', huff(16) )
CALL sub_init_huff( 8, 14, '00001011000100', huff(16) )
CALL sub_init_huff( 8, 15,     '0000000111', huff(16) )
!
CALL sub_init_huff( 9,  0,    '00010011010', huff(16) )
CALL sub_init_huff( 9,  1,     '0001001100', huff(16) )
CALL sub_init_huff( 9,  2,     '0001001001', huff(16) )
CALL sub_init_huff( 9,  3,    '00010001101', huff(16) )
CALL sub_init_huff( 9,  4,    '00010000011', huff(16) )
CALL sub_init_huff( 9,  5,   '000100000000', huff(16) )
CALL sub_init_huff( 9,  6,   '000011110101', huff(16) )
CALL sub_init_huff( 9,  7,  '0000110101010', huff(16) )
CALL sub_init_huff( 9,  8,  '0000110010110', huff(16) )
CALL sub_init_huff( 9,  9,  '0000110001010', huff(16) )
CALL sub_init_huff( 9, 10,  '0000110000000', huff(16) )
CALL sub_init_huff( 9, 11, '00001011011111', huff(16) )
CALL sub_init_huff( 9, 12,  '0000101100111', huff(16) )
CALL sub_init_huff( 9, 13, '00001011000110', huff(16) )
CALL sub_init_huff( 9, 14,  '0000101100000', huff(16) )
CALL sub_init_huff( 9, 15,    '00000001011', huff(16) )
!
CALL sub_init_huff(10,  0,     '00010001011', huff(16) )
CALL sub_init_huff(10,  1,     '00010000001', huff(16) )
CALL sub_init_huff(10,  2,      '0001000011', huff(16) )
CALL sub_init_huff(10,  3,     '00001111101', huff(16) )
CALL sub_init_huff(10,  4,    '000011110111', huff(16) )
CALL sub_init_huff(10,  5,    '000011101001', huff(16) )
CALL sub_init_huff(10,  6,    '000011100101', huff(16) )
CALL sub_init_huff(10,  7,    '000011011011', huff(16) )
CALL sub_init_huff(10,  8,   '0000110001001', huff(16) )
CALL sub_init_huff(10,  9,  '00001011100111', huff(16) )
CALL sub_init_huff(10, 10,  '00001011100001', huff(16) )
CALL sub_init_huff(10, 11,  '00001011010000', huff(16) )
CALL sub_init_huff(10, 12, '000001101110101', huff(16) )
CALL sub_init_huff(10, 13, '000001101110010', huff(16) )
CALL sub_init_huff(10, 14,  '00000110110111', huff(16) )
CALL sub_init_huff(10, 15,      '0000000100', huff(16) )
!
CALL sub_init_huff(11,  0,   '000011110011', huff(16) )
CALL sub_init_huff(11,  1,    '00001111000', huff(16) )
CALL sub_init_huff(11,  2,    '00001110110', huff(16) )
CALL sub_init_huff(11,  3,    '00001110011', huff(16) )
CALL sub_init_huff(11,  4,   '000011100011', huff(16) )
CALL sub_init_huff(11,  5,   '000011011111', huff(16) )
CALL sub_init_huff(11,  6,  '0000110001100', huff(16) )
CALL sub_init_huff(11,  7, '00001011101010', huff(16) )
CALL sub_init_huff(11,  8, '00001011100110', huff(16) )
CALL sub_init_huff(11,  9, '00001011100000', huff(16) )
CALL sub_init_huff(11, 10, '00001011010001', huff(16) )
CALL sub_init_huff(11, 11, '00001011001000', huff(16) )
CALL sub_init_huff(11, 12, '00001011000010', huff(16) )
CALL sub_init_huff(11, 13,  '0000011011111', huff(16) )
CALL sub_init_huff(11, 14, '00000110110100', huff(16) )
CALL sub_init_huff(11, 15,    '00000000110', huff(16) )
!
CALL sub_init_huff(12,  0,     '000011001010', huff(16) )
CALL sub_init_huff(12,  1,     '000011100000', huff(16) )
CALL sub_init_huff(12,  2,     '000011011110', huff(16) )
CALL sub_init_huff(12,  3,     '000011011010', huff(16) )
CALL sub_init_huff(12,  4,     '000011011000', huff(16) )
CALL sub_init_huff(12,  5,    '0000110000101', huff(16) )
CALL sub_init_huff(12,  6,    '0000110000010', huff(16) )
CALL sub_init_huff(12,  7,    '0000101111101', huff(16) )
CALL sub_init_huff(12,  8,    '0000101101100', huff(16) )
CALL sub_init_huff(12,  9,  '000001101111000', huff(16) )
CALL sub_init_huff(12, 10,   '00000110111011', huff(16) )
CALL sub_init_huff(12, 11,   '00001011000011', huff(16) )
CALL sub_init_huff(12, 12,   '00000110111000', huff(16) )
CALL sub_init_huff(12, 13,   '00000110110101', huff(16) )
CALL sub_init_huff(12, 14, '0000011011000000', huff(16) )
CALL sub_init_huff(12, 15,      '00000000100', huff(16) )
!
CALL sub_init_huff(13,  0,    '00001011101011', huff(16) )
CALL sub_init_huff(13,  1,      '000011010011', huff(16) )
CALL sub_init_huff(13,  2,      '000011010010', huff(16) )
CALL sub_init_huff(13,  3,      '000011010000', huff(16) )
CALL sub_init_huff(13,  4,     '0000101110010', huff(16) )
CALL sub_init_huff(13,  5,     '0000101111011', huff(16) )
CALL sub_init_huff(13,  6,    '00001011011110', huff(16) )
CALL sub_init_huff(13,  7,    '00001011010011', huff(16) )
CALL sub_init_huff(13,  8,    '00001011001010', huff(16) )
CALL sub_init_huff(13,  9,  '0000011011000111', huff(16) )
CALL sub_init_huff(13, 10,   '000001101110011', huff(16) )
CALL sub_init_huff(13, 11,   '000001101101101', huff(16) )
CALL sub_init_huff(13, 12,   '000001101101100', huff(16) )
CALL sub_init_huff(13, 13, '00000110110000011', huff(16) )
CALL sub_init_huff(13, 14,   '000001101100001', huff(16) )
CALL sub_init_huff(13, 15,       '00000000010', huff(16) )
!
CALL sub_init_huff(14,  0,     '0000101111001', huff(16) )
CALL sub_init_huff(14,  1,     '0000101110001', huff(16) )
CALL sub_init_huff(14,  2,       '00001100110', huff(16) )
CALL sub_init_huff(14,  3,      '000010111011', huff(16) )
CALL sub_init_huff(14,  4,    '00001011010110', huff(16) )
CALL sub_init_huff(14,  5,    '00001011010010', huff(16) )
CALL sub_init_huff(14,  6,     '0000101100110', huff(16) )
CALL sub_init_huff(14,  7,    '00001011000111', huff(16) )
CALL sub_init_huff(14,  8,    '00001011000101', huff(16) )
CALL sub_init_huff(14,  9,   '000001101100010', huff(16) )
CALL sub_init_huff(14, 10,  '0000011011000110', huff(16) )
CALL sub_init_huff(14, 11,   '000001101100111', huff(16) )
CALL sub_init_huff(14, 12, '00000110110000010', huff(16) )
CALL sub_init_huff(14, 13,   '000001101100110', huff(16) )
CALL sub_init_huff(14, 14,    '00000110110010', huff(16) )
CALL sub_init_huff(14, 15,       '00000000000', huff(16) )
!
CALL sub_init_huff(15,  0,   '000001100', huff(16) )
CALL sub_init_huff(15,  1,    '00001010', huff(16) )
CALL sub_init_huff(15,  2,    '00000111', huff(16) )
CALL sub_init_huff(15,  3,   '000001011', huff(16) )
CALL sub_init_huff(15,  4,   '000001010', huff(16) )
CALL sub_init_huff(15,  5,  '0000010001', huff(16) )
CALL sub_init_huff(15,  6,  '0000001011', huff(16) )
CALL sub_init_huff(15,  7,  '0000001001', huff(16) )
CALL sub_init_huff(15,  8, '00000001101', huff(16) )
CALL sub_init_huff(15,  9, '00000001100', huff(16) )
CALL sub_init_huff(15, 10, '00000001010', huff(16) )
CALL sub_init_huff(15, 11, '00000000111', huff(16) )
CALL sub_init_huff(15, 12, '00000000101', huff(16) )
CALL sub_init_huff(15, 13, '00000000011', huff(16) )
CALL sub_init_huff(15, 14, '00000000001', huff(16) )
CALL sub_init_huff(15, 15,    '00000011', huff(16) )
!
huff(17)%nmax    = 15
huff(17)%linbits = 2
CALL alloc_huff( huff(17) )
huff(17)%leng    = huff(16)%leng
huff(17)%icod    = huff(16)%icod
!
huff(18)%nmax    = 15
huff(18)%linbits = 3
CALL alloc_huff( huff(18) )
huff(18)%leng    = huff(16)%leng
huff(18)%icod    = huff(16)%icod
!
huff(19)%nmax    = 15
huff(19)%linbits = 4
CALL alloc_huff( huff(19) )
huff(19)%leng    = huff(16)%leng
huff(19)%icod    = huff(16)%icod
!
huff(20)%nmax    = 15
huff(20)%linbits = 6
CALL alloc_huff( huff(20) )
huff(20)%leng    = huff(16)%leng
huff(20)%icod    = huff(16)%icod
!
huff(21)%nmax    = 15
huff(21)%linbits = 8
CALL alloc_huff( huff(21) )
huff(21)%leng    = huff(16)%leng
huff(21)%icod    = huff(16)%icod
!
huff(22)%nmax    = 15
huff(22)%linbits = 10
CALL alloc_huff( huff(22) )
huff(22)%leng    = huff(16)%leng
huff(22)%icod    = huff(16)%icod
!
huff(23)%nmax    = 15
huff(23)%linbits = 13
CALL alloc_huff( huff(23) )
huff(23)%leng    = huff(16)%leng
huff(23)%icod    = huff(16)%icod
!
huff(24)%nmax    = 15
huff(24)%linbits = 4
CALL alloc_huff( huff(24) )
CALL sub_init_huff( 0,  0,         '1111', huff(24) )
CALL sub_init_huff( 0,  1,         '1101', huff(24) )
CALL sub_init_huff( 0,  2,       '101110', huff(24) )
CALL sub_init_huff( 0,  3,      '1010000', huff(24) )
CALL sub_init_huff( 0,  4,     '10010010', huff(24) )
CALL sub_init_huff( 0,  5,    '100000110', huff(24) )
CALL sub_init_huff( 0,  6,    '011111000', huff(24) )
CALL sub_init_huff( 0,  7,   '0110110010', huff(24) )
CALL sub_init_huff( 0,  8,   '0110101010', huff(24) )
CALL sub_init_huff( 0,  9,  '01010011101', huff(24) )
CALL sub_init_huff( 0, 10,  '01010001101', huff(24) )
CALL sub_init_huff( 0, 11,  '01010001001', huff(24) )
CALL sub_init_huff( 0, 12,  '01001101101', huff(24) )
CALL sub_init_huff( 0, 13,  '01000000101', huff(24) )
CALL sub_init_huff( 0, 14, '010000001000', huff(24) )
CALL sub_init_huff( 0, 15,    '001011000', huff(24) )
!
CALL sub_init_huff( 1,  0,       '1110', huff(24) )
CALL sub_init_huff( 1,  1,       '1100', huff(24) )
CALL sub_init_huff( 1,  2,      '10101', huff(24) )
CALL sub_init_huff( 1,  3,     '100110', huff(24) )
CALL sub_init_huff( 1,  4,    '1000111', huff(24) )
CALL sub_init_huff( 1,  5,   '10000010', huff(24) )
CALL sub_init_huff( 1,  6,   '01111010', huff(24) )
CALL sub_init_huff( 1,  7,  '011011000', huff(24) )
CALL sub_init_huff( 1,  8,  '011010001', huff(24) )
CALL sub_init_huff( 1,  9,  '011000110', huff(24) )
CALL sub_init_huff( 1, 10, '0101000111', huff(24) )
CALL sub_init_huff( 1, 11, '0101011001', huff(24) )
CALL sub_init_huff( 1, 12, '0100111111', huff(24) )
CALL sub_init_huff( 1, 13, '0100101001', huff(24) )
CALL sub_init_huff( 1, 14, '0100010111', huff(24) )
CALL sub_init_huff( 1, 15,   '00101010', huff(24) )
!
CALL sub_init_huff( 2,  0,      '101111', huff(24) )
CALL sub_init_huff( 2,  1,       '10110', huff(24) )
CALL sub_init_huff( 2,  2,      '101001', huff(24) )
CALL sub_init_huff( 2,  3,     '1001010', huff(24) )
CALL sub_init_huff( 2,  4,     '1000100', huff(24) )
CALL sub_init_huff( 2,  5,    '10000000', huff(24) )
CALL sub_init_huff( 2,  6,    '01111000', huff(24) )
CALL sub_init_huff( 2,  7,   '011011101', huff(24) )
CALL sub_init_huff( 2,  8,   '011001111', huff(24) )
CALL sub_init_huff( 2,  9,   '011000010', huff(24) )
CALL sub_init_huff( 2, 10,   '010110110', huff(24) )
CALL sub_init_huff( 2, 11,  '0101010100', huff(24) )
CALL sub_init_huff( 2, 12,  '0100111011', huff(24) )
CALL sub_init_huff( 2, 13,  '0100100111', huff(24) )
CALL sub_init_huff( 2, 14, '01000011101', huff(24) )
CALL sub_init_huff( 2, 15,     '0010010', huff(24) )
!
CALL sub_init_huff( 3,  0,    '1010001', huff(24) )
CALL sub_init_huff( 3,  1,     '100111', huff(24) )
CALL sub_init_huff( 3,  2,    '1001011', huff(24) )
CALL sub_init_huff( 3,  3,    '1000110', huff(24) )
CALL sub_init_huff( 3,  4,   '10000110', huff(24) )
CALL sub_init_huff( 3,  5,   '01111101', huff(24) )
CALL sub_init_huff( 3,  6,   '01110100', huff(24) )
CALL sub_init_huff( 3,  7,  '011011100', huff(24) )
CALL sub_init_huff( 3,  8,  '011001100', huff(24) )
CALL sub_init_huff( 3,  9,  '010111110', huff(24) )
CALL sub_init_huff( 3, 10,  '010110010', huff(24) )
CALL sub_init_huff( 3, 11, '0101000101', huff(24) )
CALL sub_init_huff( 3, 12, '0100110111', huff(24) )
CALL sub_init_huff( 3, 13, '0100100101', huff(24) )
CALL sub_init_huff( 3, 14, '0100001111', huff(24) )
CALL sub_init_huff( 3, 15,    '0010000', huff(24) )
!
CALL sub_init_huff( 4,  0,    '10010011', huff(24) )
CALL sub_init_huff( 4,  1,     '1001000', huff(24) )
CALL sub_init_huff( 4,  2,     '1000101', huff(24) )
CALL sub_init_huff( 4,  3,    '10000111', huff(24) )
CALL sub_init_huff( 4,  4,    '01111111', huff(24) )
CALL sub_init_huff( 4,  5,    '01110110', huff(24) )
CALL sub_init_huff( 4,  6,    '01110000', huff(24) )
CALL sub_init_huff( 4,  7,   '011010010', huff(24) )
CALL sub_init_huff( 4,  8,   '011001000', huff(24) )
CALL sub_init_huff( 4,  9,   '010111100', huff(24) )
CALL sub_init_huff( 4, 10,  '0101100000', huff(24) )
CALL sub_init_huff( 4, 11,  '0101000011', huff(24) )
CALL sub_init_huff( 4, 12,  '0100110010', huff(24) )
CALL sub_init_huff( 4, 13,  '0100011101', huff(24) )
CALL sub_init_huff( 4, 14, '01000011100', huff(24) )
CALL sub_init_huff( 4, 15,     '0001110', huff(24) )
!
CALL sub_init_huff( 5,  0,  '100000111', huff(24) )
CALL sub_init_huff( 5,  1,    '1000010', huff(24) )
CALL sub_init_huff( 5,  2,   '10000001', huff(24) )
CALL sub_init_huff( 5,  3,   '01111110', huff(24) )
CALL sub_init_huff( 5,  4,   '01110111', huff(24) )
CALL sub_init_huff( 5,  5,   '01110010', huff(24) )
CALL sub_init_huff( 5,  6,  '011010110', huff(24) )
CALL sub_init_huff( 5,  7,  '011001010', huff(24) )
CALL sub_init_huff( 5,  8,  '011000000', huff(24) )
CALL sub_init_huff( 5,  9,  '010110100', huff(24) )
CALL sub_init_huff( 5, 10, '0101010101', huff(24) )
CALL sub_init_huff( 5, 11, '0100111101', huff(24) )
CALL sub_init_huff( 5, 12, '0100101101', huff(24) )
CALL sub_init_huff( 5, 13, '0100011001', huff(24) )
CALL sub_init_huff( 5, 14, '0100000110', huff(24) )
CALL sub_init_huff( 5, 15,    '0001100', huff(24) )
!
CALL sub_init_huff( 6,  0,   '011111001', huff(24) )
CALL sub_init_huff( 6,  1,    '01111011', huff(24) )
CALL sub_init_huff( 6,  2,    '01111001', huff(24) )
CALL sub_init_huff( 6,  3,    '01110101', huff(24) )
CALL sub_init_huff( 6,  4,    '01110001', huff(24) )
CALL sub_init_huff( 6,  5,   '011010111', huff(24) )
CALL sub_init_huff( 6,  6,   '011001110', huff(24) )
CALL sub_init_huff( 6,  7,   '011000011', huff(24) )
CALL sub_init_huff( 6,  8,   '010111001', huff(24) )
CALL sub_init_huff( 6,  9,  '0101011011', huff(24) )
CALL sub_init_huff( 6, 10,  '0101001010', huff(24) )
CALL sub_init_huff( 6, 11,  '0100110100', huff(24) )
CALL sub_init_huff( 6, 12,  '0100100011', huff(24) )
CALL sub_init_huff( 6, 13,  '0100010000', huff(24) )
CALL sub_init_huff( 6, 14, '01000001000', huff(24) )
CALL sub_init_huff( 6, 15,     '0001010', huff(24) )
!
CALL sub_init_huff( 7,  0,  '0110110011', huff(24) )
CALL sub_init_huff( 7,  1,    '01110011', huff(24) )
CALL sub_init_huff( 7,  2,    '01101111', huff(24) )
CALL sub_init_huff( 7,  3,    '01101101', huff(24) )
CALL sub_init_huff( 7,  4,   '011010011', huff(24) )
CALL sub_init_huff( 7,  5,   '011001011', huff(24) )
CALL sub_init_huff( 7,  6,   '011000100', huff(24) )
CALL sub_init_huff( 7,  7,   '010111011', huff(24) )
CALL sub_init_huff( 7,  8,  '0101100001', huff(24) )
CALL sub_init_huff( 7,  9,  '0101001100', huff(24) )
CALL sub_init_huff( 7, 10,  '0100111001', huff(24) )
CALL sub_init_huff( 7, 11,  '0100101010', huff(24) )
CALL sub_init_huff( 7, 12,  '0100011011', huff(24) )
CALL sub_init_huff( 7, 13, '01000010011', huff(24) )
CALL sub_init_huff( 7, 14, '00101111101', huff(24) )
CALL sub_init_huff( 7, 15,    '00010001', huff(24) )
!
CALL sub_init_huff( 8,  0,  '0110101011', huff(24) )
CALL sub_init_huff( 8,  1,   '011010100', huff(24) )
CALL sub_init_huff( 8,  2,   '011010000', huff(24) )
CALL sub_init_huff( 8,  3,   '011001101', huff(24) )
CALL sub_init_huff( 8,  4,   '011001001', huff(24) )
CALL sub_init_huff( 8,  5,   '011000001', huff(24) )
CALL sub_init_huff( 8,  6,   '010111010', huff(24) )
CALL sub_init_huff( 8,  7,   '010110001', huff(24) )
CALL sub_init_huff( 8,  8,   '010101001', huff(24) )
CALL sub_init_huff( 8,  9,  '0101000000', huff(24) )
CALL sub_init_huff( 8, 10,  '0100101111', huff(24) )
CALL sub_init_huff( 8, 11,  '0100011110', huff(24) )
CALL sub_init_huff( 8, 12,  '0100001100', huff(24) )
CALL sub_init_huff( 8, 13, '01000000010', huff(24) )
CALL sub_init_huff( 8, 14, '00101111001', huff(24) )
CALL sub_init_huff( 8, 15,    '00010000', huff(24) )
!
CALL sub_init_huff( 9,  0,  '0101001111', huff(24) )
CALL sub_init_huff( 9,  1,   '011000111', huff(24) )
CALL sub_init_huff( 9,  2,   '011000101', huff(24) )
CALL sub_init_huff( 9,  3,   '010111111', huff(24) )
CALL sub_init_huff( 9,  4,   '010111101', huff(24) )
CALL sub_init_huff( 9,  5,   '010110101', huff(24) )
CALL sub_init_huff( 9,  6,   '010101110', huff(24) )
CALL sub_init_huff( 9,  7,  '0101001101', huff(24) )
CALL sub_init_huff( 9,  8,  '0101000001', huff(24) )
CALL sub_init_huff( 9,  9,  '0100110001', huff(24) )
CALL sub_init_huff( 9, 10,  '0100100001', huff(24) )
CALL sub_init_huff( 9, 11,  '0100010011', huff(24) )
CALL sub_init_huff( 9, 12, '01000001001', huff(24) )
CALL sub_init_huff( 9, 13, '00101111011', huff(24) )
CALL sub_init_huff( 9, 14, '00101110011', huff(24) )
CALL sub_init_huff( 9, 15,    '00001011', huff(24) )
!
CALL sub_init_huff(10,  0, '01010011100', huff(24) )
CALL sub_init_huff(10,  1,   '010111000', huff(24) )
CALL sub_init_huff(10,  2,   '010110111', huff(24) )
CALL sub_init_huff(10,  3,   '010110011', huff(24) )
CALL sub_init_huff(10,  4,   '010101111', huff(24) )
CALL sub_init_huff(10,  5,  '0101011000', huff(24) )
CALL sub_init_huff(10,  6,  '0101001011', huff(24) )
CALL sub_init_huff(10,  7,  '0100111010', huff(24) )
CALL sub_init_huff(10,  8,  '0100110000', huff(24) )
CALL sub_init_huff(10,  9,  '0100100010', huff(24) )
CALL sub_init_huff(10, 10,  '0100010101', huff(24) )
CALL sub_init_huff(10, 11, '01000010010', huff(24) )
CALL sub_init_huff(10, 12, '00101111111', huff(24) )
CALL sub_init_huff(10, 13, '00101110101', huff(24) )
CALL sub_init_huff(10, 14, '00101101110', huff(24) )
CALL sub_init_huff(10, 15,    '00001010', huff(24) )
!
CALL sub_init_huff(11,  0, '01010001100', huff(24) )
CALL sub_init_huff(11,  1,  '0101011010', huff(24) )
CALL sub_init_huff(11,  2,   '010101011', huff(24) )
CALL sub_init_huff(11,  3,   '010101000', huff(24) )
CALL sub_init_huff(11,  4,   '010100100', huff(24) )
CALL sub_init_huff(11,  5,  '0100111110', huff(24) )
CALL sub_init_huff(11,  6,  '0100110101', huff(24) )
CALL sub_init_huff(11,  7,  '0100101011', huff(24) )
CALL sub_init_huff(11,  8,  '0100011111', huff(24) )
CALL sub_init_huff(11,  9,  '0100010100', huff(24) )
CALL sub_init_huff(11, 10,  '0100000111', huff(24) )
CALL sub_init_huff(11, 11, '01000000001', huff(24) )
CALL sub_init_huff(11, 12, '00101110111', huff(24) )
CALL sub_init_huff(11, 13, '00101110000', huff(24) )
CALL sub_init_huff(11, 14, '00101101010', huff(24) )
CALL sub_init_huff(11, 15,    '00000110', huff(24) )
!
CALL sub_init_huff(12,  0, '01010001000', huff(24) )
CALL sub_init_huff(12,  1,  '0101000010', huff(24) )
CALL sub_init_huff(12,  2,  '0100111100', huff(24) )
CALL sub_init_huff(12,  3,  '0100111000', huff(24) )
CALL sub_init_huff(12,  4,  '0100110011', huff(24) )
CALL sub_init_huff(12,  5,  '0100101110', huff(24) )
CALL sub_init_huff(12,  6,  '0100100100', huff(24) )
CALL sub_init_huff(12,  7,  '0100011100', huff(24) )
CALL sub_init_huff(12,  8,  '0100001101', huff(24) )
CALL sub_init_huff(12,  9,  '0100000101', huff(24) )
CALL sub_init_huff(12, 10, '01000000000', huff(24) )
CALL sub_init_huff(12, 11, '00101111000', huff(24) )
CALL sub_init_huff(12, 12, '00101110010', huff(24) )
CALL sub_init_huff(12, 13, '00101101100', huff(24) )
CALL sub_init_huff(12, 14, '00101100111', huff(24) )
CALL sub_init_huff(12, 15,    '00000100', huff(24) )
!
CALL sub_init_huff(13,  0, '01001101100', huff(24) )
CALL sub_init_huff(13,  1,  '0100101100', huff(24) )
CALL sub_init_huff(13,  2,  '0100101000', huff(24) )
CALL sub_init_huff(13,  3,  '0100100110', huff(24) )
CALL sub_init_huff(13,  4,  '0100100000', huff(24) )
CALL sub_init_huff(13,  5,  '0100011010', huff(24) )
CALL sub_init_huff(13,  6,  '0100010001', huff(24) )
CALL sub_init_huff(13,  7,  '0100001010', huff(24) )
CALL sub_init_huff(13,  8, '01000000011', huff(24) )
CALL sub_init_huff(13,  9, '00101111100', huff(24) )
CALL sub_init_huff(13, 10, '00101110110', huff(24) )
CALL sub_init_huff(13, 11, '00101110001', huff(24) )
CALL sub_init_huff(13, 12, '00101101101', huff(24) )
CALL sub_init_huff(13, 13, '00101101001', huff(24) )
CALL sub_init_huff(13, 14, '00101100101', huff(24) )
CALL sub_init_huff(13, 15,    '00000010', huff(24) )
!
CALL sub_init_huff(14,  0, '010000001001', huff(24) )
CALL sub_init_huff(14,  1,   '0100011000', huff(24) )
CALL sub_init_huff(14,  2,   '0100010110', huff(24) )
CALL sub_init_huff(14,  3,   '0100010010', huff(24) )
CALL sub_init_huff(14,  4,   '0100001011', huff(24) )
CALL sub_init_huff(14,  5,   '0100001000', huff(24) )
CALL sub_init_huff(14,  6,   '0100000011', huff(24) )
CALL sub_init_huff(14,  7,  '00101111110', huff(24) )
CALL sub_init_huff(14,  8,  '00101111010', huff(24) )
CALL sub_init_huff(14,  9,  '00101110100', huff(24) )
CALL sub_init_huff(14, 10,  '00101101111', huff(24) )
CALL sub_init_huff(14, 11,  '00101101011', huff(24) )
CALL sub_init_huff(14, 12,  '00101101000', huff(24) )
CALL sub_init_huff(14, 13,  '00101100110', huff(24) )
CALL sub_init_huff(14, 14,  '00101100100', huff(24) )
CALL sub_init_huff(14, 15,     '00000000', huff(24) )
!
CALL sub_init_huff(15,  0, '00101011', huff(24) )
CALL sub_init_huff(15,  1,  '0010100', huff(24) )
CALL sub_init_huff(15,  2,  '0010011', huff(24) )
CALL sub_init_huff(15,  3,  '0010001', huff(24) )
CALL sub_init_huff(15,  4,  '0001111', huff(24) )
CALL sub_init_huff(15,  5,  '0001101', huff(24) )
CALL sub_init_huff(15,  6,  '0001011', huff(24) )
CALL sub_init_huff(15,  7,  '0001001', huff(24) )
CALL sub_init_huff(15,  8,  '0000111', huff(24) )
CALL sub_init_huff(15,  9,  '0000110', huff(24) )
CALL sub_init_huff(15, 10,  '0000100', huff(24) )
CALL sub_init_huff(15, 11, '00000111', huff(24) )
CALL sub_init_huff(15, 12, '00000101', huff(24) )
CALL sub_init_huff(15, 13, '00000011', huff(24) )
CALL sub_init_huff(15, 14, '00000001', huff(24) )
CALL sub_init_huff(15, 15,     '0011', huff(24) )
!
huff(25)%nmax    = 15
huff(25)%linbits = 5
CALL alloc_huff( huff(25) )
huff(25)%leng    = huff(24)%leng
huff(25)%icod    = huff(24)%icod
!
huff(26)%nmax    = 15
huff(26)%linbits = 6
CALL alloc_huff( huff(26) )
huff(26)%leng    = huff(24)%leng
huff(26)%icod    = huff(24)%icod
!
huff(27)%nmax    = 15
huff(27)%linbits = 7
CALL alloc_huff( huff(27) )
huff(27)%leng    = huff(24)%leng
huff(27)%icod    = huff(24)%icod
!
huff(28)%nmax    = 15
huff(28)%linbits = 8
CALL alloc_huff( huff(28) )
huff(28)%leng    = huff(24)%leng
huff(28)%icod    = huff(24)%icod
!
huff(29)%nmax    = 15
huff(29)%linbits = 9
CALL alloc_huff( huff(29) )
huff(29)%leng    = huff(24)%leng
huff(29)%icod    = huff(24)%icod
!
huff(30)%nmax    = 15
huff(30)%linbits = 11
CALL alloc_huff( huff(30) )
huff(30)%leng    = huff(24)%leng
huff(30)%icod    = huff(24)%icod
!
huff(31)%nmax    = 15
huff(31)%linbits = 13
CALL alloc_huff( huff(31) )
huff(31)%leng = huff(24)%leng
huff(31)%icod = huff(24)%icod
!
RETURN
END SUBROUTINE init_huffman
!----------------------------------------------------------------------------------------- 
SUBROUTINE alloc_huff(huff)
IMPLICIT NONE
TYPE (huff_), INTENT(IN OUT) :: huff
INTEGER :: n
n = huff%nmax
huff%linmax = 2**huff%linbits - 1
ALLOCATE( huff%leng(0:n, 0:n), huff%icod(0:n, 0:n) )
RETURN
END SUBROUTINE alloc_huff
!----------------------------------------------------------------------------------------- 
SUBROUTINE sub_init_huff(i, j, c_code, huff)
IMPLICIT NONE
INTEGER            , INTENT(IN    ) :: i, j
CHARACTER (LEN = *), INTENT(IN    ) :: c_code
TYPE (huff_)       , INTENT(IN OUT) :: huff
huff%leng(i, j) = LEN_TRIM(c_code)
READ(c_code, '(B20.0)') huff%icod(i, j)
!print *, i, j, huff%icod(i, j), LEN_TRIM(c_code), "----", c_code
!pause
RETURN
END SUBROUTINE sub_init_huff
!----------------------------------------------------------------------------------------- 
SUBROUTINE init_huffman_quadruples()
IMPLICIT NONE
huff_qa(0,0,0,0)%leng = 1
huff_qa(0,0,0,1)%leng = 4
huff_qa(0,0,1,0)%leng = 4
huff_qa(0,0,1,1)%leng = 5
huff_qa(0,1,0,0)%leng = 4
huff_qa(0,1,0,1)%leng = 6
huff_qa(0,1,1,0)%leng = 5
huff_qa(0,1,1,1)%leng = 6
huff_qa(1,0,0,0)%leng = 4
huff_qa(1,0,0,1)%leng = 5
huff_qa(1,0,1,0)%leng = 5
huff_qa(1,0,1,1)%leng = 6
huff_qa(1,1,0,0)%leng = 5
huff_qa(1,1,0,1)%leng = 6
huff_qa(1,1,1,0)%leng = 6
huff_qa(1,1,1,1)%leng = 6
!
huff_qa(0,0,0,0)%icod = 1 ! B'1'
huff_qa(0,0,0,1)%icod = 5 ! B'0101'
huff_qa(0,0,1,0)%icod = 4 ! B'0100'
huff_qa(0,0,1,1)%icod = 5 ! B'00101'
huff_qa(0,1,0,0)%icod = 6 ! B'0110'
huff_qa(0,1,0,1)%icod = 5 ! B'000101'
huff_qa(0,1,1,0)%icod = 4 ! B'00100'
huff_qa(0,1,1,1)%icod = 4 ! B'000100'
huff_qa(1,0,0,0)%icod = 7 ! B'0111'
huff_qa(1,0,0,1)%icod = 3 ! B'00011'
huff_qa(1,0,1,0)%icod = 6 ! B'00110'
huff_qa(1,0,1,1)%icod = 0 ! B'000000'
huff_qa(1,1,0,0)%icod = 7 ! B'00111'
huff_qa(1,1,0,1)%icod = 2 ! B'000010'
huff_qa(1,1,1,0)%icod = 3 ! B'000011'
huff_qa(1,1,1,1)%icod = 1 ! B'000001'
!
huff_qb(0,0,0,0)%leng = 4
huff_qb(0,0,0,1)%leng = 4
huff_qb(0,0,1,0)%leng = 4
huff_qb(0,0,1,1)%leng = 4
huff_qb(0,1,0,0)%leng = 4
huff_qb(0,1,0,1)%leng = 4
huff_qb(0,1,1,0)%leng = 4
huff_qb(0,1,1,1)%leng = 4
huff_qb(1,0,0,0)%leng = 4
huff_qb(1,0,0,1)%leng = 4
huff_qb(1,0,1,0)%leng = 4
huff_qb(1,0,1,1)%leng = 4
huff_qb(1,1,0,0)%leng = 4
huff_qb(1,1,0,1)%leng = 4
huff_qb(1,1,1,0)%leng = 4
huff_qb(1,1,1,1)%leng = 4
!
huff_qb(0,0,0,0)%icod = 15 ! B'1111'
huff_qb(0,0,0,1)%icod = 14 ! B'1110'
huff_qb(0,0,1,0)%icod = 13 ! B'1101'
huff_qb(0,0,1,1)%icod = 12 ! B'1100'
huff_qb(0,1,0,0)%icod = 11 ! B'1011'
huff_qb(0,1,0,1)%icod = 10 ! B'1010'
huff_qb(0,1,1,0)%icod =  9 ! B'1001'
huff_qb(0,1,1,1)%icod =  8 ! B'1000'
huff_qb(1,0,0,0)%icod =  7 ! B'0111'
huff_qb(1,0,0,1)%icod =  6 ! B'0110'
huff_qb(1,0,1,0)%icod =  5 ! B'0101'
huff_qb(1,0,1,1)%icod =  4 ! B'0100'
huff_qb(1,1,0,0)%icod =  3 ! B'0011'
huff_qb(1,1,0,1)%icod =  2 ! B'0010'
huff_qb(1,1,1,0)%icod =  1 ! B'0001'
huff_qb(1,1,1,1)%icod =  0 ! B'0000'
RETURN
END SUBROUTINE init_huffman_quadruples
END MODULE mod_huffman
