MODULE mod_psycho
USE mod_mpg
IMPLICIT NONE
PRIVATE
PUBLIC :: psycho      ! subroutine
PUBLIC :: ath_l, ath_s, ath_m ! variable
REAL (KIND = 8), SAVE :: ath_l(32, 18), ath_s(32, 18), ath_m(32, 18)
LOGICAL        , SAVE :: qfirst = .true.
CONTAINS
!----------------------------------------------------------------
FUNCTION switch_q(subband) RESULT(ires)
IMPLICIT NONE
REAL (KIND = 8), INTENT(IN) :: subband(:, :, :)
INTEGER :: ires
REAL (KIND = 8), SAVE :: sum0a, sum1a = EPSILON(0.0d0), sum0b, sum1b = EPSILON(0.0d0)
sum0a = sum1a
sum1a = SUM(subband(1: 2, :, :)**2) + EPSILON(0.0d0) 
sum0b = sum1b
sum1b = SUM(subband(3:n_switch_max, :, :)**2) + EPSILON(0.0d0) 
IF ( sum1a / sum0a >= switch .OR. sum1b / sum0b >= switch ) THEN
 ires = mblock_type_param
 IF (q_sm .AND. sum1a / sum0a < 1.1d0) ires = 21 ! mixed 
ELSE 
 ires = 0 
END IF
IF (mblock_type_param == 0) ires = 0 ! force long-only mode
! debug info
if ( sum1a / sum0a >= switch  .OR.  sum0a / sum1a > switch ) nn1 = nn1 + 1
if ( sum1b / sum0b >= switch  .OR.  sum0b / sum1b > switch ) nn2 = nn2 + 1
RETURN
END FUNCTION switch_q
!----------------------------------------------------------------
SUBROUTINE psycho(mpg, mblock_type, subband)
IMPLICIT NONE
TYPE (mpeg_parameters), INTENT(IN OUT) :: mpg
INTEGER               , INTENT(   OUT) :: mblock_type(:, :)
REAL (KIND = 8)       , INTENT(IN    ) :: subband(:, :, :)
INTEGER, SAVE :: mblock_prev = 0
INTEGER :: i, m, iattack
real (kind = 8):: pm(2), pm0
DO i = 1, 2
 m = 18 * (i - 1) + 1 
 iattack = switch_q(subband(:, m:m + 17, :)) 
 SELECT CASE (iattack)
  CASE (0) ! no-attack
    SELECT CASE (mblock_prev)
      CASE ( 0, 30, 31) ! long
        mblock_type(i, :) =  0
      CASE (10) 
        mblock_type(i, :) = 20
      CASE (11) 
        mblock_type(i, :) = 21
      CASE (20) ! short
        mblock_type(i, :) = 30
      CASE (21) ! mixed
        mblock_type(i, :) = 31
      CASE DEFAULT
        WRITE(*, *) 'error: psycho : unexpected block type: case 0-x', i, mblock_prev, mblock_type(i, 1) 
        STOP
      END SELECT
  CASE (20) ! attack-short
    SELECT CASE (mblock_prev)
      CASE ( 0, 30, 31)  
        mblock_type(i, :) = 10
      CASE (10) 
        mblock_type(i, :) = 20
      CASE (11) 
        mblock_type(i, :) = 21
      CASE (20) 
        mblock_type(i, :) = 20
      CASE (21) 
        mblock_type(i, :) = 21 
      CASE DEFAULT
        WRITE(*, *) 'error: psycho : unexpected block type: case 20-x', i, mblock_prev, mblock_type(i, 1) 
        STOP
      END SELECT
  CASE (21) ! attack-mixed
    SELECT CASE (mblock_prev)
      CASE ( 0, 30, 31)  
        mblock_type(i, :) = 11
      CASE (10) 
        mblock_type(i, :) = 20
      CASE (11) 
        mblock_type(i, :) = 21
      CASE (20) 
        mblock_type(i, :) = 20 
      CASE (21) 
        mblock_type(i, :) = 21
      CASE DEFAULT
        WRITE(*, *) 'error: psycho : unexpected block type: case 21-x', i, mblock_prev, mblock_type(i, 1) 
        STOP
      END SELECT
  CASE DEFAULT
    WRITE(*, *) 'error: psycho : unexpected block type'
    STOP
 END SELECT
 mblock_prev = mblock_type(i, 1)
! debug info
 select case ( mblock_type(i, 1) )
  case ( 0) 
   long = long + 1
  case (20)
   nshort = nshort + 1
  case (21) 
   mix = mix + 1
  case (10, 11)
   m1 = m1 + 1
  case (30, 31)
   m3 = m3 + 1
 end select
!
END DO
!..... test ...............................................................
!Psychoacoustic Moment
IF (q_vbr) THEN
 PM = 0.0d0
 DO i = 1, 32
  pm(1) = pm(1) + SUM( subband(i, :, 1)**2 ) * REAL(i, KIND = 8) 
  pm(2) = pm(2) + SUM( subband(i, :, 2)**2 ) * REAL(i, KIND = 8) 
 END DO
 pm(1) = pm(1) / SUM(subband(:, :, 1)**2 + EPSILON(0.0d0))
 pm(2) = pm(2) / SUM(subband(:, :, 2)**2 + EPSILON(0.0d0))
 pm0 = SUM(pm) * 0.5d0
 IF (pm0 < 0.95d0) then
   mpg%ibit_rate =  1 
 ELSE IF (pm0 * pm_factor < 1.05d0) then
   mpg%ibit_rate =  8 
 ELSE IF (pm0 * pm_factor < 1.3d0) then
   mpg%ibit_rate =  9 
 ELSE IF (pm0 * pm_factor < 4.0d0) then
   mpg%ibit_rate = 10 
 ELSE IF (pm0 * pm_factor < 6.0d0) then
   mpg%ibit_rate = 11 
 ELSE 
   mpg%ibit_rate = 12
 END IF
 IF ( mblock_type(1, 1) /= 0 .OR. mblock_type(2, 1) /= 0 ) mpg%ibit_rate = mpg%ibit_rate + 1
END IF
nbits(mpg%ibit_rate) = nbits(mpg%ibit_rate) + 1
!!!- to avoid VBR bug of portable MP3 player DIAMOND MULTIMEDIA RIO500.---------------------------------
IF (qfirst) THEN
 IF (q_vbr .and. q_rio500) mpg%ibit_rate = 1 ! force first frame 32kbps for RIO500 (firmware 1.15)  
 qfirst = .false.
 CALL init_absolute_threshold(mpeg_sample_rates(mpg%isample_rate) )
END IF
!!!-----------------------------------------------------------------------------------------------------
RETURN
END SUBROUTINE psycho
!------------------------------------------------------------------------------------------------
SUBROUTINE init_absolute_threshold(isample_rate)
IMPLICIT NONE
INTEGER, INTENT(IN ) :: isample_rate
REAL(KIND = 8):: freq, temp, ath(576)
INTEGER :: i, iwin, iband
DO i = 1, 576
 freq = REAL(isample_rate, KIND = 8) / 2.0d0 / 1000.0d0 &
      * (REAL(i, KIND = 8) - 0.0d0) / 575.0d0
 temp = 3.64d0  * freq ** (-0.8d0) & 
      - 6.50d0  * EXP(-0.6d0 * (freq -  3.3d0)**2.0d0) &
      + 0.001d0 * freq ** 4.0d0      
 temp = MIN(temp, 70.0d0) + ath_min 
 ath(i) = 10.0d0**(temp / 20.0d0) 
END DO
!
DO iband = 1, 32
 DO i = 1, 18
  ath_l(iband, i) = ath( 18 * (iband - 1) + i)
 END DO
END DO
DO iband = 1, 32
 DO iwin = 1, 3 
  DO i = 1, 6
   ath_s(iband, 6 * (iwin - 1) + i) = ath( 18 * (iband - 1) + 3 * i )
  END DO
 END DO
END DO
ath_m(1: 2, :) = ath_l(1: 2, :)
ath_m(3:32, :) = ath_s(3:32, :)
RETURN
END SUBROUTINE init_absolute_threshold
!------------------------------------------------------------------------------------------------
END MODULE mod_psycho